# slicer-build

This repository contains build scripts used to build Cura and all dependencies from scratch. Currently we only support Windows x64 builds.


## Upstream information
Upstream repository: [Ultimaker/cura-build](https://github.com/Ultimaker/cura-build)
Last commit from upstream: _cea66e6_

## Windows

### Dependencies
It is strongly advised to build [slicer-build-environment](https://212.106.159.105:81/svn/3D_Gen/build-system/slicer-build-environment) first which will provide most dependencies.

Following things have to be installed on windows system:

* **git for windows** (https://git-for-windows.github.io/) The `git` command should be available on your `%PATH%`. Make sure that the `cmd` directory in the git for windows installation directory is on the `%PATH%` and *not* its `bin` directory, otherwise mingw32 will complain about `sh.exe` being on the path.
* **CMake** (http://www.cmake.org/) Once CMake is installed make sure it is available on your `%PATH%`. Check this by running `cmake --version` in the Windows console.
* **MinGW-W64** >= 5.3.0 (http://mingw-w64.org/doku.php) Once installed, its `bin` directory should be available on your `%PATH%`. Test this by running `mingw32-make --version` in the Windows console.
* **Microsoft Visual Studio 2015 (community edition)**: 
  Install Programming languages: Visual c++ (all), Python Tools for Visual Studio (Nov 2015)
  Windows & Web Development: Universal Windows App Development Tools (Tools 1.2 & windows 10 SDK-10/0/10586; Windows 10 SDK -10.0.10240) - ___It MUST be 2015 version of Visual Studio!___
* **NSIS 3** (http://nsis.sourceforge.net/Main_Page) for creating the installer. Additional NSIS scripts are shipped with this project.

Make sure these dependencies are available from your path.

Additionally, for 32-bit builds:

* Perl (http://www.activestate.com/activeperl, Required to build Qt)
* Create in the user directory a file named pydistutils.cfg with the following contents:
```shell
[build]
compiler=mingw32
```

### Building
#### For builds with slicer-build-environment
Copy directory described as ___$INSTALL\_DIR___ created during slicer-build-environment build to _slicer-build/build/inst_

#### Building 64-bit version
```shell
REM 64-bit
git clone git@github.com:Ultimaker/cura-build.git
cd cura-build
mkdir build
cd build
..\env_win64.bat
cmake -G "MinGW Makefiles" -DBUILD_64BIT:BOOL=ON -DSIGN_PACKAGE:BOOL=OFF -DCMAKE_PREFIX_PATH="path_to_inst_dir" ..
for example :
cmake -G "MinGW Makefiles" -DBUILD_64BIT:BOOL=ON -DSIGN_PACKAGE:BOOL=ON  -DSIGN_INSTALLER:BOOL=ON -DCMAKE_PREFIX_PATH="C:\Users\softint_user\Desktop\dev_build\slicer-build\build\inst" -DCURA_VERSION_MAJOR="1" -DCURA_VERSION_MINOR="1" -DCURA_VERSION_PATCH="1" ..
mingw32-make
mingw32-make package
mingw32-make signing-installer
```
Example cmake invocation:
```shell
cmake -G "MinGW Makefiles" -DBUILD_64BIT:BOOL=ON -DSIGN_PACKAGE:BOOL=OFF -DCMAKE_PREFIX_PATH="C:\Users\SoftInt_Kamil\Desktop\3DGence_build\cura-build\build\inst" ..
```

Currently __mingw32-make package__ doesn't work :)
#### Building 32-bit version - unsupported

```shell
REM 32-bit
git clone git@github.com:Ultimaker/cura-build.git
cd cura-build
mkdir build
cd build
..\env_win32.bat
cmake -G "MinGW Makefiles" ..
mingw32-make
mingw32-make package
```

### Additional steps
1. Copy _repo/src/Cura/Cura/cura_ and _repo/src/Uranium/UM_ into _slicer-build/build/package_
2. Copy _python.exe_ and _cura\_app.py_ from _slicer-build\build\inst\bin_ into _slicer-build\build\package_
3. Build _repo/src/Runner2_ in QTCreator
4. Copy _slicer-build/build/package_ into program created in previous point and rename it to _resources_
5. Copy folder from previous point into _repo/src/Installer_ and build _3DGence Slicer Installer Script.nsi_ using ___NSIS3___

# Unsupported systems
## Ubuntu/Linux

cura-build can build AppImage packages of Cura.

To build, make sure these dependencies are installed, then clone this repository and run the following commands from your clone:

```shell
git clone http://github.com/Ultimaker/cura-build.git
cd cura-build
```

```shell
mkdir build
cd build
cmake ..
make
make package
```

## OS X

1. Install CMake (available via [homebrew](http://brew.sh/) or [cmake.org](http://www.cmake.org/))
2. Install latest version of Xcode.
3. On Mac OS X > 10.10, execute command: *brew link openssl --force*
4. Because Fortran is necessary: *brew install gcc*
5. Run these commands:
```shell
git clone git@github.com:Ultimaker/cura-build.git
cd cura-build
mkdir build
cd build
cmake ..
make
```
