import math

from cura.SIProxies.AutoarrangePlugin.OccupiedChunk import OccupiedChunk
from cura.SIProxies.AutoarrangePlugin.ModelLines import ModelLines
from cura.SIProxies.AutoarrangePlugin.Toolbox import *

class Autoarranger:
    def __init__(self, size, px_chunk_size, mm_chunk_size, mm_gap_size):
        self.width = size[0]
        self.depth = size[1]
        self.px_chunk_size = px_chunk_size
        self.autoarrange_gap_size = mm_gap_size/mm_chunk_size
        self.chunks_width = math.floor(self.width/mm_chunk_size)
        self.chunks_height = math.floor(self.depth/mm_chunk_size)

        self.rows = list(list() for i in range(self.chunks_height))
        self.columns = list(list() for i in range(self.chunks_width))

    def __place_model(self, model_lines):
        finished = False
        tick_phase = True #algorithm is combined from two phases, tick and tock (up- and downward move)
        diagonal_move = False
        max_x = self.chunks_width - model_lines.width - 1
        max_y = self.chunks_height - model_lines.height - 1
        final_x_reached = False
        final_y_reached = False
        move_right = (1, 0)
        move_up = (0, -1)
        move_diag_down = (1, 1)
        move_diag_up = (-1, -1)

        if max_x < 0 or max_y < 0:
            return False

        model_lines.y_shift = max_y

        if self.__check_position_validity(model_lines):
            self.__add_model_lines(model_lines)
            return True

        while not finished:
            if diagonal_move:
                if tick_phase:
                    model_lines.move_model(move_diag_down)
                    if (final_x_reached and model_lines.x_shift == max_x) or model_lines.y_shift == max_y:
                        tick_phase = False
                        diagonal_move = False
                else:
                    model_lines.move_model(move_diag_up)
                    if (final_y_reached and model_lines.y_shift == 0) or model_lines.x_shift == 0:
                        tick_phase = True
                        diagonal_move = False

            else:
                diagonal_move = True
                if tick_phase:
                    model_lines.move_model(move_right if final_x_reached else move_up)
                else:
                    model_lines.move_model(move_up if final_y_reached else move_right)

            current_position = model_lines.get_position()

            if not final_x_reached and model_lines.x_shift == max_x:
                final_x_reached = True

            if not final_y_reached and model_lines.y_shift == 0:
                final_y_reached = True

            if model_lines.x_shift == max_x and model_lines.y_shift == 0:
                finished = True

            if self.__check_position_validity(model_lines):
                self.__add_model_lines(model_lines)
                return True

        return False
 
    def __check_position_validity(self, model_lines):
        for i, row in enumerate(model_lines.lines):
            for model_vector in row:
                field_row_number = model_lines.y_shift + i
                for existing_line in self.rows[field_row_number]:
                    vector_to_test = model_vector+model_lines.x_shift
                    if check_overlapping(existing_line, vector_to_test, self.autoarrange_gap_size):
                        return False

        for i, column in enumerate(model_lines.columns):
            for model_vector in column:
                field_column_number = model_lines.x_shift + i
                for existing_line in self.columns[field_column_number]:
                    vector_to_test = model_vector+model_lines.y_shift
                    if check_overlapping(existing_line, vector_to_test, self.autoarrange_gap_size):
                        return False

        return True

    def __add_model_lines(self, model_lines):
        for i, row in enumerate(model_lines.lines):
            for chunk in row:
                chunk_to_add = chunk+model_lines.x_shift
                self.add_chunk_to_row(model_lines.y_shift+i, chunk_to_add)

        for i, column in enumerate(model_lines.columns):
            for chunk in column:
                chunk_to_add = chunk+model_lines.y_shift
                self.add_chunk_to_column(model_lines.x_shift+i, chunk_to_add)

    def add_chunk_to_row(self, row, chunk):
        self.rows[row].append(chunk)
        self.rows[row].sort()

    def add_chunk_to_column(self, column, chunk):
        self.columns[column].append(chunk)
        self.columns[column].sort()

    def add_model(self, model):
        chunked_model = chunk_model(model, self.px_chunk_size)
        model_lines = ModelLines(chunked_model.shape)
        model_lines.from_bool_array(chunked_model)
        if self.__place_model(model_lines):
            return (model_lines.x_shift, model_lines.y_shift, model_lines.width, model_lines.height)
        else:
            return False

    def add_models_list(self, models_list):
        result_list = list()
        for model in models_list:
            model_result = self.add_model(model[1])
            result_list.append((model[0], model_result))

        return result_list