# Copyright (c) 2015 Ultimaker B.V.
# Uranium is released under the terms of the AGPLv3 or higher.

from PyQt5.QtCore import QObject, pyqtSlot, QVariant
# from PyQt5.QtWidgets import QAction
# from PyQt5.QtQml import QQmlListProperty
from UM.Math.Quaternion import Quaternion
from UM.Math.Vector import Vector

# from ListField import ListField
# from PrintersSource import PrintersSource
# from TypesSource import TypesSource
# from MaterialsSource import MaterialsSource
# from QualitiesSource import QualitiesSource
# from DurabilitiesSource import DurabilitiesSource

# from UM.PluginRegistry import PluginRegistry
from UM.Application import Application
from UM.Scene.Camera import Camera

# from UM.Mesh.MeshBuilder import MeshBuilder
# from UM.Math.Color import Color
from UM.Scene.SceneNode import SceneNode
# from UM.Scene.Scene import Scene

# from UM.Math.Matrix import Matrix
from cura.BuildVolume import BuildVolume

from UM.Scene.Selection import Selection

import math


class CameraOperator(QObject):
    """
    Class visible in QML file as SI.CameraOperator

    """
    instance = None
    # Distance allows to make sure whole object is visisble in fieldview
    distance = 4
    ######################### INIT #########################
    ########################################################
    # "width,height,depth of BuildVolume to set camera position"

    def __init__(self, camera: Camera, width, height, depth, volume: BuildVolume):
        super().__init__(None)


        if (camera == None):
            return

        self._scene = Application.getInstance().getController().getScene()
        self._tool = Application.getInstance().getController().getTool("CameraTool")
        self._camera = camera
        self._camera.setPerspective(True)
        self._scene.setActiveCamera("3d")
        self._volume = volume
        self._currentPerspective = 'front'
        self._table_width = width

        # mb = MeshBuilder()
        # mb.addCube(
        #     width=0.04,
        #     height=0.001,
        #     depth=0.001,
        #     center=Vector(0, 0, 0),
        #     color=Color(255, 0, 0, 255)
        # )
        # mb.addCube(
        #     width=0.001,
        #     height=0.04,
        #     depth=0.001,
        #     center=Vector(-0.04 / 2, 0.04 / 2, 0),
        #     color=Color(255, 0, 0, 255)
        # )
        # mb.addCube(
        #     width=0.001,
        #     height=0.001,
        #     depth=0.04,
        #     center=Vector(-0.04 / 2, 0, -0.04 / 2),
        #     color=Color(255, 0, 0, 255)
        # )

        # node = SceneNode(self._camera)

        # node._origin_mesh = mb.build()
        # node.setMeshData(node._origin_mesh)

        # node.setPosition(Vector(0.15, -0.12, -0.1))

        self._height = height
        onBias = Vector(depth * CameraOperator.distance, height * 2, width)
        self._regularPositions = {
            'front': Vector(0, height*0.3, width * CameraOperator.distance),
            'left': Vector(-depth * CameraOperator.distance, height*0.5, 0),
            'top': Vector(0, width * CameraOperator.distance, 1),
            'leftFrontUpper': Vector(-1, 1, 1) * onBias,
            'rightFrontUpper': Vector(1, 1, 1) * onBias,
            'leftBackUpper': Vector(-1, 1, -1) * onBias,
            'rightBackUpper': Vector(1, 1, -1) * onBias,
            'leftBackBottom': Vector(-1, -1, -1) * onBias,
            'leftFrontBottom': Vector(-1, -1, 1) * onBias,
            'rightFrontBottom': Vector(1, -1, 1) * onBias,
        }

        self._reversedPositions = {
            'front': Vector(0, height*0.3, -width * CameraOperator.distance),
            'left': Vector(depth * CameraOperator.distance, height*0.5, 0),
            'top': Vector(0, width * CameraOperator.distance, -1),
            'leftFrontUpper': Vector(1, 1, -1) * onBias,
            'rightFrontUpper': Vector(-1, 1, -1) * onBias,
            'leftBackUpper': Vector(1, 1, 1) * onBias,
            'rightBackUpper': Vector(-1, 1, 1) * onBias,
            'leftBackBottom': Vector(1, -1, 1) * onBias,
            'leftFrontBottom': Vector(1, -1, -1) * onBias,
            'rightFrontBottom': Vector(-1, -1, -1) * onBias,
        }

        self._positions = self._regularPositions


        CameraOperator.instance = self
        self._camera.setPosition(self._positions['front'])
        self.showFrom('front')

        # self._camera.transformationChanged.connect(self.test);

    def test(self, arg: SceneNode):
        pass
        #
        # if(not type(arg) == Camera):
        #     self._camera.transformationChanged.disconnect(self.test)
        #
        #     buildVolume = self._volume
        #     axisOnVolume = buildVolume._axis
        #
        #     orginalTransformation = axisOnVolume.getWorldTransformation()
        #     orginalRotation = orginalTransformation.getEuler()
        #
        #     copyTransformation = arg.getWorldTransformation()
        #     copyTransformation.setByRotationAxis(orginalRotation.y,Vector.Unit_Y)
        #
        #     arg.setTransformation(copyTransformation)
        #     self._camera.transformationChanged.connect(self.test)
        #

    @pyqtSlot(QVariant)
    def showFrom(self, pointOfView):
        position = self._positions[pointOfView]
        if (position != None):
            self._currentPerspective = pointOfView
            # Set rotation point
            self._tool.setOrigin(Vector(0, self._height / 2, 0))
            self._camera.setPosition(position)
            self._camera.setZoom(0.0)
            mainWindow = Application.getInstance().getMainWindow()
            if(mainWindow):
                mainWindow.updateViewportGeometry()
            self._camera.lookAt(Vector(0, self._height / 2, 0))
            self._tool.setToTheSameDistanceinOrth(self._table_width)

    @pyqtSlot()
    def reset(self):
        self.showFrom(self._currentPerspective)

    @pyqtSlot()
    def center(self):
        selected = Selection.getAllSelectedObjects()
        if (selected):
            node = selected[0]
            nodePosition = node.getPosition()
            self._tool.setOrigin(nodePosition)
            nodeBoundingBox = node.getBoundingBox()
            nodeMaxSize = max(nodeBoundingBox.depth, nodeBoundingBox.width, nodeBoundingBox.height)
            self._camera.setPosition(nodePosition + Vector(-1.3, 1.3, 1.3)*nodeMaxSize)
            self._camera.lookAt(nodePosition)

            mainWindow = Application.getInstance().getMainWindow()
            if(mainWindow):
                mainWindow.updateViewportGeometry()
            
            self._tool.setToTheSameDistanceinOrth(self._table_width)
        else:
            self.showFrom("leftFrontUpper")

    # Flip camera (for reversed plate printers)
    def reverseCamera(self):
        currentPosition = self._camera.getPosition()
        rotation = Quaternion.fromAngleAxis(math.pi, Vector.Unit_Y)
        self._camera.rotate(rotation, SceneNode.TransformSpace.World)
        self._camera.setPosition(Vector(-currentPosition.x, currentPosition.y, -currentPosition.z))

        if self._positions == self._regularPositions:
            self._positions = self._reversedPositions
        else:
            self._positions = self._regularPositions

def createDataCameraOperator(engine = None, script_engine = None):
    if (CameraOperator.instance == None):
        CameraOperator.instance = CameraOperator(None, None, None, None)

    return CameraOperator.instance
