'''
Class created to gather all models together, with only one instance per class,
this way we does not have to create many singletons
'''

from PyQt5.QtCore import QObject, QVariant, pyqtProperty, pyqtSignal, pyqtSlot

from UM.Application import Application

from . import (DurabilitiesModel, MaterialsModel, NozzlesModel,
               PrintableNodesModel, QualitiesModel)


class ModelsHolder(QObject):
    '''
    Class created to gather all models together, with only one instance per class,
    this way we does not have to create many singletons, in PyQT 30 is the maximum
    '''

    def __init__(self, parent=None):
        if Application.getInstance()._models != None:
            raise ValueError("Duplicate singleton creation of ModelsHolder")
        Application.getInstance()._models = self

        super().__init__(parent)

        self._materials = MaterialsModel.CreateMaterialsModel(None, None)
        self._nozzles = NozzlesModel.CreateNozzlesModel(None, None)
        self._qualities = QualitiesModel.CreateQualitiesModel(None, None)
        self._durabilities = DurabilitiesModel.CreateDurabilitiesModel(None, None)
        self._printable_nodes = PrintableNodesModel.CreatePrintableNodesModel(None, None)

    materialsSignal = pyqtSignal(str, arguments=["materials"])
    nozzlesSignal = pyqtSignal(str, arguments=["nozzles"])
    qualitiesSignal = pyqtSignal(str, arguments=["qualities"])
    durabilitiesSignal = pyqtSignal(str, arguments=["durabilities"])
    printableNodesSignal = pyqtSignal(str, arguments=["printableNodes"])

    @pyqtProperty(MaterialsModel.MaterialsModel, notify=materialsSignal)
    def materials(self):
        return self._materials

    @pyqtProperty(NozzlesModel.NozzlesModel, notify=nozzlesSignal)
    def nozzles(self):
        return self._nozzles

    @pyqtProperty(QualitiesModel.QualitiesModel, notify=qualitiesSignal)
    def qualities(self):
        return self._qualities

    @pyqtProperty(QualitiesModel.QualitiesModel, notify=qualitiesSignal)
    def durabilities(self):
        return self._durabilities

    @pyqtProperty(PrintableNodesModel.PrintableNodesModel, notify=printableNodesSignal)
    def printableNodes(self):
        return self._printable_nodes

    @classmethod
    def getInstance(cls):
        if Application.getInstance()._models is None:
            Application.getInstance()._models = cls()

        return Application.getInstance()._models


def CreateModelsHolder(engine, script_engine):
    return ModelsHolder.getInstance()
