from UM.Preferences import Preferences
from cura.SIProxies.SettingsManage.PrintersTree import PrinterTree
from cura.SIProxies.SettingsManage.PrinterSettingsDB import PrinterSettingsDB
from cura.View.RightMenuManager import RightMenuManager
import os

class ProfileLoaderManager:
    _instance = None

    def __init__(self):
        self._is_overridden = None
        self._current_printer = None
        self._profile_settings = None
        self._profile_set_manually = False

        self._gcode_profile_changed = None
        self._gcode_current_profile = None

    def force_set_profile(self, new_profile: dict):
        self._profile_set_manually = True

        prepared_profile = dict()
        prepared_profile['Printer'] = new_profile['printer']
        prepared_profile['Nozzle'] = new_profile['nozzle']
        prepared_profile['Material'] = new_profile['material']
        prepared_profile['Quality'] = new_profile['quality']
        prepared_profile['Durability'] = new_profile['durability']

        self._profile_settings = prepared_profile

        # As in QML - doesn't work as normal function
        from cura.SIProxies.SettingsManage.NotificationsProxy import NotificationsProxy
        NotificationsProxy.GetInstance().printer = prepared_profile['Printer']

    def special_case(self, new_printer: str):
        if not self._profile_set_manually:
            # First load after program start
            if self._current_printer is None:
                self._profile_settings = self.get_last_loaded()
                self._current_printer = self._profile_settings['Printer']
            else:
                # We want to load default profile for new printer
                self._profile_settings = self.get_default_leaf(new_printer)
                self._current_printer = new_printer

        self._is_overridden = True
        
        return True

    ## Sets default profile for provided printer and profile
    def set_gcode_profile(self, current_profile, printer_name: str = None):
        if printer_name:
            db = PrinterSettingsDB()
            profile = db.get_default_profile_by_printer(printer_name)
            PrinterTree.getInstance().onDurabilitySelected(profile[4], profile[3], profile[2], profile[1], profile[0])

            self._gcode_profile_changed = True
            names = [(printer_name, False), ("Default", False), ("Default", False), ("Default", False), ("Default", False)]
        else:
            self._gcode_profile_changed = False
            names = [("Unrecognized", False), ("Unrecognized", False), ("Unrecognized", False), ("Unrecognized", False), ("Unrecognized", False)]

        RightMenuManager.GetInstance().set_profiles_ui(names)

        self._gcode_current_profile = current_profile

    def unset_gcode_profile(self):
        profile_to_load = []
        for setting in self._gcode_current_profile:
            profile_to_load.append((setting, True))

        if self._gcode_profile_changed:
            PrinterTree.getInstance().onDurabilitySelected(profile_to_load[4][0],
                                                           profile_to_load[3][0],
                                                           profile_to_load[2][0],
                                                           profile_to_load[1][0],
                                                           profile_to_load[0][0])

        RightMenuManager.GetInstance().set_profiles_ui(profile_to_load)

        self._gcode_profile_changed = None
        self._gcode_current_profile = None

    def get_special_case(self):
        if self._is_overridden is None:
            return False
        return self._is_overridden

    def set_special_case(self, value: bool = False):
        self._is_overridden = value
        self._profile_set_manually = False
        self._profile_settings = False

    def get_special_parameter(self, key: str):
        result = None
        if self._profile_settings is not None:
            if self._profile_settings[key] is not None:
                result = self._profile_settings[key]

        return result

    def get_last_loaded(self):
        profile_id = Preferences.getInstance().getValue("cura/last_profile_loaded")

        raw_data = None
        db = PrinterSettingsDB()

        # Try to load profile if it's active
        if profile_id is not None and profile_id != "None":
            raw_data = db.get_profile_by_id(profile_id)


        # Fallback if there is no last used profile or if profile is inactive
        if raw_data is None:
            raw_data = db.get_default_profile_by_printer('Industry')

        if raw_data is None:
            raw_data = db.get_first_active_profile()

        db.close()

        prepared_profile = dict()

        prepared_profile['Printer'] = raw_data[0]
        prepared_profile['Nozzle'] = raw_data[1]
        prepared_profile['Material'] = raw_data[2]
        prepared_profile['Quality'] = raw_data[3]
        prepared_profile['Durability'] = raw_data[4]

        return prepared_profile

    def get_default_leaf(self, printer: str):
        db = PrinterSettingsDB()

        raw_data = db.get_default_profile_by_printer(printer)

        db.close()

        prepared_profile = dict()

        prepared_profile['Printer'] = raw_data[0]
        prepared_profile['Nozzle'] = raw_data[1]
        prepared_profile['Material'] = raw_data[2]
        prepared_profile['Quality'] = raw_data[3]
        prepared_profile['Durability'] = raw_data[4]

        return prepared_profile

    def set_last_used(self, printer: str, nozzle: str, material: str, quality: str, durability: str):
        db = PrinterSettingsDB()
        profile_id = db.get_profile_id_from_options(printer, nozzle, material, quality, durability)
        db.close()

        Preferences.getInstance().setValue("cura/last_profile_loaded", profile_id)
        return

    def set_printer_by_real_name(self, printer_file_name: str):
        db = PrinterSettingsDB()

        printer_id = db.get_printer_id_by_real_name(printer_file_name)

        if printer_id is False:
            return False

        profile = db.get_default_profile_by_printer_id(printer_id)

        if profile is False:
            profile = db.get_any_profile_by_printer_id(printer_id)

        if profile is False:
            return False

        db.close()

        prepared_profile = dict()

        prepared_profile['Printer'] = profile[0]
        prepared_profile['Nozzle'] = profile[1]
        prepared_profile['Material'] = profile[2]
        prepared_profile['Quality'] = profile[3]
        prepared_profile['Durability'] = profile[4]

        self._profile_settings = prepared_profile

        self._profile_set_manually = True

        from cura.SIProxies.SettingsManage.NotificationsProxy import NotificationsProxy
        NotificationsProxy.GetInstance().printer = prepared_profile['Printer']

    @classmethod
    def GetInstance(cls):
        if ProfileLoaderManager._instance is None:
            ProfileLoaderManager._instance = cls()

        return ProfileLoaderManager._instance
