import sys
import requests

from UM.Job import Job
from PyQt5.QtCore import QObject
from cura.Updater.UpdateData import UpdateData
from cura.Updater.Status import Status
from UM.Application import Application
from UM.Preferences import Preferences

class ServerClient(QObject, Job):

    __serverDomain = 'http://check.autoupdate.3dgence.com:8889/'
    __updateArgs = []
    __errorCode = 0

    # Class constructor
    def __init__(self):
        super().__init__()
        self.__app = Application.getInstance()

    # Set error code
    def __set_error_code(self, errorCode):
        self.__errorCode = errorCode

    # Get error code
    def __get_error_code(self):
        return self.__errorCode

    # Set update arguments
    def setArgs(self, update_arg = None):
        self.__updateArgs = update_arg

    #Run thread
    def run(self):
        self.__update(self.__updateArgs[0], self.__updateArgs[1], self.__updateArgs[2],
                      self.__updateArgs[3], self.__updateArgs[4])

    # Update
    def __update(self, applicationVersion, activeProfiles, activePrinters, activeExtruders, fdmVersion):

        # Tylko dla testów
        include_test_profiles  = Preferences.getInstance().getValue("general/include_test_profiles") \
            if Preferences.getInstance().getValue("general/include_test_profiles") is not None else False
        include_draft_profiles = Preferences.getInstance().getValue("general/include_draft_profiles") \
            if Preferences.getInstance().getValue("general/include_draft_profiles") is not None else False

        params   = {'application_version': applicationVersion,
                    'active_profiles': activeProfiles,
                    'active_printers': activePrinters,
                    'active_extruders': activeExtruders,
                    'fdm_version': fdmVersion,
                    'include_test_profiles': include_test_profiles, # Tylko dla testów
                    'include_draft_profiles': include_draft_profiles} # Tylko dla testów

        response = self.__send_request('version/check/', params)

        if response is not None:
            data = UpdateData()
            data.set_data_from_json(response.text)
            if data.get_data() is not None:
                status = data.parse_data()
                if status != 5 and status != 14:
                    if self.__get_error_code() == 0:
                        data.save_to_file()
                    self.__update_status_message(status)
                else:
                    self.__app.displayStatus(Status().get_status_description(status))
        else:
            self.__update_status_message(15)

    # Download update information
    def __update_status_message (self, update_state):
        status = Status()
        errorCode = self.__get_error_code()

        if errorCode == 0:
            if update_state == 3:
                self.__app.displayStatus(status.get_status_description(3))
            else:
                self.__app.displayStatus(status.get_status_description(2))
        else:
            self.__app.displayStatus(status.get_status_description(update_state, errorCode))

    # Map status response to error code
    def __map_status_response_to_error(self, responseStatus):
        return {
            400: 4,
            404: 4,
            413: 2,
            422: 2,
            500: 2
        }[responseStatus]

    # Send request
    def __send_request(self, endpoint, params):

        response = None

        try:
            response = requests.post(self.__serverDomain + endpoint, json=params)

            if response.status_code != 200:
                ServerClient.setErrorCode(self.__map_status_response_to_error(response.status_code))
                response = None

            return response

        except requests.exceptions.SSLError as exception:
            self.__set_error_code(8)
            print('Exception : SSLError! \nDescription : ', exception)
            return response
        except requests.exceptions.ReadTimeout as exception:
            self.__set_error_code(5)
            print('Exception : MaxRetryError! \nDescription : ', exception)
            return response
        except requests.exceptions.ReadTimeout as exception:
            self.__set_error_code(3)
            print('Exception: ReadTimeoutError! \nDescription : ', exception)
            return response
        except requests.exceptions.Timeout as exception:
            self.__set_error_code(3)
            print('Exception: TimeoutError! \nDescription : ', exception)
            return response
        except requests.exceptions.ConnectionError as exception:
            self.__set_error_code(1)
            print('Exception: ConnectionError! \nDescription : ', exception)
            return response
        except requests.exceptions.HTTPError as exception:
            self.__set_error_code(1)
            print('Exception: HTTPError! \nDescription : ', exception)
            return response
        except:
            self.__set_error_code(4)
            print("Unexpected error:", sys.exc_info ()[0])
            return response