import os
import sys
import json
import pickle

from UM.Resources import Resources

class UpdateData:

    __file_name = 'temp_download_data.gen'

    def __init__(self):

        self.__data = None
        self.__new_fdm = None
        self.__new_printers = None
        self.__new_extruders = None
        self.__new_profiles = None
        self.__active_printers = None
        self.__active_extruders = None
        self.__active_profiles = None

        self.__path_to_data_to_update = os.path.join(Resources.getPath(Resources.GenceSettings), self.__file_name)

    def set_data_from_json(self, json_response):
        self.__data = json.loads(json_response)

    def get_data(self):
        return self.__data

    # Parse download data
    def parse_data(self):

        if self.__need_version_upgrade() is True:
            return 5

        if self.__need_db_update():
            self.__update_fdm()
            self.__update_printers()
            self.__update_extruders()
            self.__update_profiles()
            return 3
        else:
            return 14

    ## Checks parts that need update
    # Check if application is up to date.
    def __need_version_upgrade(self):
        return True if self.__data.get('application_version', None) else False

    # Check if database is up to date.
    def __need_db_update(self):
        return False if self.__data.get('db_up_to_date', None) else True

    ## Populate object with new data
    # Parse fdm.
    def __update_fdm (self):
        if self.__data.get('fdm_files_up_to_date', False) is False:
            self.__new_fdm = self.__data.get('fdm_files', None)

    def __update_printers(self):
        self.__active_printers = self.__data.get('active_printers', None)
        self.__new_printers    = self.__data.get('new_printers', None)

    def __update_extruders(self):
        self.__active_extruders = self.__data.get('active_extruders', None)
        self.__new_extruders = self.__data.get('new_extruders', None)

    def __update_profiles(self):
        self.__active_profiles = self.__data.get('active_profiles', None)
        self.__new_profiles = self.__data.get('new_profiles', None)

    ## Getters
    # Get active profiles table
    def get_active_profiles(self):
        return self.__active_profiles

    # Get profiles object table
    def get_new_profiles(self):
        return self.__new_profiles

    # Get new extruders obj
    def get_new_extruders(self):
        return self.__new_extruders

    # Get active printers table
    def get_active_printers(self):
        return self.__active_printers

    # Get active extruders
    def get_active_extruders(self):
        return self.__active_extruders

    # Get new printers object table
    def get_new_printers(self):
        return self.__new_printers

    # Get new fdm object
    def get_new_fdm(self):
        return self.__new_fdm

    ## File operations
    # Save download data to file.
    def save_to_file(self):
        file = open(self.__path_to_data_to_update, 'wb')
        pickle.dump(self.__data, file)
        file.close()

    # Read download data form file
    def load_from_file(self):
        file = open(self.__path_to_data_to_update, 'rb')
        self.__data = pickle.load(file)
        self.parse_data()
        file.close()

    # Is file exist.
    def file_exists(self):
        return os.path.isfile (self.__path_to_data_to_update)

    # Remove file.
    def remove_file(self):
        os.unlink(self.__path_to_data_to_update)
