from UM.Settings.Models.SettingPropertyProvider import SettingPropertyProvider
from UM.Application import Application

class CommentFields:
    """Keeps all keys of comment lines"""

    _plugin_id=None

    header = "header"
    machine = "Machine"
    module = "Module"
    profile = "Profile"
    material = "Material"
    quality = "Quality"
    durability="Durability"
    empty="empty"
    date="Date"
    time="Time"
    eta="ETA"
    filament0="Filament T0 used"
    filament1="Filament T1 used"
    nozzle0="Nozzle T0"
    nozzle1="Nozzle T1"
    material1="Material T0"
    material2="Material T1"
    start="Start"



    # Holds all the comments lines with values, but witchout any order, so to achive ordered
    # comments we use order_of_lines array
    commentsLines = \
        {
            header:"Generated with 3DGence Slicer " + Application.getInstance()._version,
            empty:"",
            start:"Start"
        }

    # Holds all the comments lines with values, but without any order, so to achive ordered
    # comments we use order_of_lines array
    linesOrder =\
        [
            header,
            machine,
            # module,
            # profile,
            # material,
            quality,
            durability,
            date,
            time,
            eta,
            filament0,
            filament1,
            nozzle0,
            nozzle1,
            material1,
            material2,
            start
        ]

    # ;Generated with 3DGence Slicer v1.0
    # ;Machine: Industry 340
    # ;Module: Standard
    # :Profile: Model(both extruders)
    # ;Material: ABS
    # ;Quality: High
    # ;Durability: Normal
    # ;
    # ;mar 20, 2017 at 10: 58:10
    # ;TIME: 7266
    # ;FilamentT0 used: 8.11678m
    # ;FilamentT1 used: 8.11678m
    # ;NozzleT0: 0.4
    # ;NozzleT1: 0.4
    # ;MaterialT0 used: ABS
    # ;MaterialT1 used: PLA
    # ;
    # ;Start

    # Holds all the keys which won't be printed in comment , ex ";mar 20, 2017 at 10:58:10" ,
    # not ";Date: mar 20, 2017 at 10:58:10"
    printWithoutKey=\
        {
            header,
            empty,
            # date,
            start
        }

    @classmethod
    def commentsToString(cls)->str:
        allLines=""
        array = cls.commentsToArray()
        for line in array:
            allLines += line+"\n"
        return allLines


    @classmethod
    def commentsToArray(cls)->[str]:
        array=[]
        for key in cls.linesOrder:
            value = cls.commentsLines.get(key,None)
            printWithoutKey = key in cls.printWithoutKey;

            lineAsString = ";"

            # If we do not want to print line key we write ;<value> only
            if(printWithoutKey):
                if value:
                    lineAsString += value
            else:
                if value:
                    lineAsString += key +": "+ value

            if value:
                array.append(lineAsString)


        return array


    @classmethod
    def updateComment(cls, key, value):
        cls.commentsLines[key]=value

    @classmethod
    def getUsedExtruder(cls):
        settings_provider = SettingPropertyProvider()
        globalContainerStack = Application.getInstance().getGlobalContainerStack()
        if globalContainerStack:
            stackId = globalContainerStack._id
            settings_provider.setContainerStackId(stackId)
            settings_provider.setKey("machine_start_gcode")
            start_gcode = settings_provider._getPropertyValue("value")
            splitted_start_gcode = start_gcode.split("\n")

            for line in reversed(splitted_start_gcode):
                if line == "T0":
                    return 0
                elif line == "T1":
                    return 1

        return 0

    @classmethod
    def getPluginId(cls):
        return cls._plugin_id

    @classmethod
    def setPluginId(cls, plugin_id):
        cls._plugin_id = plugin_id
