from UM.Resources import Resources
from UM.Application import Application

from UM.View.RenderPass import RenderPass
from UM.View.RenderBatch import RenderBatch
from UM.View.GL.OpenGL import OpenGL

##  A RenderPass subclass responsible for rendering selectable objects to a texture.
class PolygonSelectionPass(RenderPass):
    def __init__(self):
        super().__init__("polygon_selection", 0, 0, -999)

        self._shader = OpenGL.getInstance().createShaderProgram(Resources.getPath(Resources.Shaders, "default.shader"))#"selection.shader"))
        self._gl = OpenGL.getInstance().getBindingsObject()
        self._scene = Application.getInstance().getController().getScene()

        self._renderer = Application.getInstance().getRenderer()


        self._visible_node = None
        self._visible_node_changed = True
        self._last_camera_pos = None


    ##  Perform the actual rendering.
    def render(self):
        current_camera_pos = self._scene.getActiveCamera().getPosition()
        render = False

        if current_camera_pos != self._last_camera_pos:
            self._last_camera_pos = current_camera_pos
            render = True

        if self._visible_node_changed:
            self._visible_node_changed = False
            render = True

        if render:
            node = self._scene.findObject(self._visible_node)

            if node is not None and node.isSelectable() and node.getMeshData():
                batch = RenderBatch(self._shader)
                batch.addItem(transformation = node.getWorldTransformation(), mesh = node.getMeshData())#, uniforms = { "selection_color": self._getNodeColor(node)})

                self.bind()

                batch.render(self._scene.getActiveCamera())

                self.release()

    ##  Get the object id at a certain pixel coordinate.
    def getColorAtPosition(self, x, y):
        output = self.getOutput()

        window_size = self._renderer.getWindowSize()

        px = (0.5 + x / 2.0) * window_size[0]
        py = (0.5 + y / 2.0) * window_size[1]

        if px < 0 or px > (output.width() - 1) or py < 0 or py > (output.height() - 1):
            return None

        pixel = output.pixel(px, py)

        return pixel

    def setVisibleNode(self, node_id):
        self._visible_node = node_id
        self._visible_node_changed = True
