from UM.Scene.SceneNode import SceneNode
from UM.Mesh.MeshBuilder import MeshBuilder
from UM.Application import Application
from UM.Math.Vector import Vector
from UM.Math.Color import Color
from .SimpleSplitMesh import Axis

class GenceSpliterPlane(SceneNode):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._scene = Application.getInstance().getController().getScene()

        self.setParent(self._scene.getRoot())

        self._platform_color = Color.fromHexString("#B4FF1010")

    def showSplitPlane(self, axis, bounding_box, center):
        self.setVisible(True)
        self._calculateMesh(axis, bounding_box, center)
        self.meshDataChanged.emit(self)

    def hideSplitPlane(self):
        self.setVisible(False)
        self.meshDataChanged.emit(self)

    def setPlanePosition(self, axis, value):
        pos = self.getWorldPosition()

        if axis == Axis.X:
            new_postion = pos.set(x=value + (pos.x - self.getBoundingBox().center.x))
        if axis == Axis.Y:
            new_postion = pos.set(z=value + (pos.z - self.getBoundingBox().center.z))
        if axis == Axis.Z:
            new_postion = pos.set(y=value + (pos.y - self.getBoundingBox().bottom))

        self.setPosition(new_postion, SceneNode.TransformSpace.World)

    def _calculateMesh(self, axis, bb, center):
        self.setPosition(Vector(0, 0, 0))
        mb_plain = MeshBuilder()

        height = bb.top - bb.bottom + 20
        width = bb.right - bb.left + 20
        depth = bb.front - bb.back + 20

        if axis == Axis.X:
            mb_plain.addCube(0.1, height, depth, center ,color=self._platform_color)
        if axis == Axis.Z:
            mb_plain.addCube(width, 0.1, depth, center, color=self._platform_color)
        if axis == Axis.Y:
            mb_plain.addCube(width, height, 0.1, center, color=self._platform_color)

        self.setMeshData(mb_plain.build())
