import triangle
import numpy as np
from .Split import Split, Axis, Side


class SplitMesh(Split):
    def __init__(self, axis, value, debug):
        Split.__init__(self)

        self.__axis = axis
        self.__value = value
        self.__debug = debug

    def setSplitMeshValue(self, value):
        self.__value = value

    # TODO: Przemyśleć sprawę osobnego cięcia i liczenia dla lewej i prawej strony
    def _calculate_cut_halfs(self, vertices, normals, progress_part):
        step = int(max((len(vertices) / 300), 1))
        leftVertices = []
        rightVertices = []

        leftSegmentsNormals = []
        rightSegmentsNormals = []

        leftSectionsOnCut = []
        rightSectionsOnCut = []

        for i, vert_list in enumerate(self._grouped(vertices, 3)):
            side, vertices_sides = self._checkPolygonSide(vert_list)

            currentLeftVertices = []
            currentRightVertices = []

            if side == Side.Cut:
                currentLeftVertices, currentRightVertices = self._calculateSplittedPolygons(vert_list, vertices_sides)

                current_left_cut_sections = self._findOnCutSections(currentLeftVertices)
                current_right_cut_sections = self._findOnCutSections(currentRightVertices)

                leftSectionsOnCut.extend(current_left_cut_sections)
                rightSectionsOnCut.extend(current_right_cut_sections)

                leftSegmentsNormals += len(current_left_cut_sections) * [normals[i * 3]]
                rightSegmentsNormals += len(current_right_cut_sections) * [normals[i * 3]]

            else:
                section = None
                if vertices_sides.count(Side.OnPlane) == 2:
                    section = self._findOnCutSections(vert_list)

                if side == Side.Left:
                    currentLeftVertices = vert_list
                    if section is not None:
                        leftSectionsOnCut.extend(section)
                        leftSegmentsNormals += len(section) * [normals[i * 3]]

                if side == Side.Right:
                    currentRightVertices = vert_list
                    if section is not None:
                        rightSectionsOnCut.extend(section)
                        rightSegmentsNormals += len(section) * [normals[i * 3]]


            leftVertices.extend(currentLeftVertices)
            rightVertices.extend(currentRightVertices)

            if i % step == 0:
                self._add_node_progress((1 / 100) * progress_part)

        return {
            'left_vertices': leftVertices,
            'right_vertices': rightVertices,

            'left_segments_normals': leftSegmentsNormals,
            'right_segments_normals': rightSegmentsNormals,

            'left_sections_on_cut': leftSectionsOnCut,
            'right_sections_on_cut': rightSectionsOnCut
        }

    def _checkPolygonSide(self, polygon):
        vert_1_side = self._checkVertexSide(polygon[0])
        vert_2_side = self._checkVertexSide(polygon[1])
        vert_3_side = self._checkVertexSide(polygon[2])

        vertices_sides = [vert_1_side, vert_2_side, vert_3_side]
        side = self._polygonSide(vertices_sides)

        return side, vertices_sides

    def _checkVertexSide(self, vertex):
        vertex_side = Side.Left

        if self.__value < vertex[self.__axis]:
            vertex_side = Side.Right
        elif self.__value == vertex[self.__axis]:
            vertex_side = Side.OnPlane

        return vertex_side

    def _polygonSide(self, vertices_sides):
        current_side = None

        for vertex_side in vertices_sides:
            if current_side is None:
                current_side = vertex_side
            elif vertex_side == Side.OnPlane:
                continue
            elif vertex_side == current_side:
                continue
            elif current_side == Side.OnPlane:
                current_side = vertex_side
            else:
                current_side = Side.Cut
                break

        return current_side

    def _findOnCutSections(self, polygon_list):
        sections_on_cut = []
        for vert_0, vert_1, vert_2 in self._grouped(polygon_list, 3):
            current_sections = []
            if vert_0[self.__axis] == self.__value and vert_1[self.__axis] == self.__value:
                current_sections.append([vert_0, vert_1])

            if vert_0[self.__axis] == self.__value and vert_2[self.__axis] == self.__value:
                current_sections.append([vert_2, vert_0])

            if vert_1[self.__axis] == self.__value and vert_2[self.__axis] == self.__value:
                current_sections.append([vert_1, vert_2])

            current_sections_2d = []
            for section in current_sections:
                current_sections_2d.append(np.delete(section, self.__axis, 1))

            sections_on_cut.extend(current_sections_2d)

        return sections_on_cut

    def _prepare3dPolygons(self, sections_on_cut, sections_normals):
        sections_normals = np.delete(sections_normals, self.__axis, 1)
        shapes = self._findShapesOnCut(sections_on_cut, sections_normals)

        closing_2d = self._calculateClosingPolygons(shapes)

        closing_3d = []
        for polygon_2d in closing_2d:
            polygon_3d = list()
            polygon_3d.append(polygon_2d[0])
            polygon_3d.append(polygon_2d[1])
            polygon_3d.insert(self.__axis, self.__value)
            closing_3d.append(np.asarray(polygon_3d))

        return closing_3d
    
    def _findShapesOnCut(self, sections, normals):
        shapes = []

        sections = self._removeInvalidSections(sections)

        normals_copy = list(normals)

        while sections:
            shape = [sections[0]]
            start = sections[0][0]
            current = sections[0][1]

            sections.pop(0)
            normal = normals_copy.pop(0)

            while not self._v_eq(start, current):
                for i, section in enumerate(sections):
                    if self._v_eq(current, section[0]):
                        current = section[1]
                        shape.append(section)
                        sections.pop(i)
                        normals_copy.pop(i)
                        break
                else:
                    print("Patologiczny przypadek")
                    break

            if 2 < len(shape):
                # To do: [0, 1] [1, 2] [2, 3] [3, 1] [1, 0]
                shape_info = {}
                shape_info['fill'] = self._isFilled(normal, shape)
                shape_info['segments'] = shape
                shapes.append(shape_info)
            else:
                print("tak samo patola")

        self.__debug_printShapes(shapes)

        return shapes
    
    def _removeInvalidSections(self, sections):
        valid_sections = []

        for section in sections:
            if not self._v_eq(section[0], section[1]):
                valid_sections.append(section)

        return valid_sections

    def _isFilled(self, normal, shape):
        cut_points = []
        #We want to test in the middle of the section
        p1 = np.asarray([(shape[0][0][0] + shape[0][1][0])/2, (shape[0][0][1] + shape[0][1][1])/2])
        p2 = np.asarray([p1[0] + normal[0], p1[1] + normal[1]])

        #hole_point = np.asarray([p1[0] + normal[0] * 0.0001, p1[1] + normal[1] * 0.0001])

        a, b, c = self._lineFromTwoPoints(p1, p2)

        #Check if big if() needed
        cut_count = 0
        if a == 1 and b == 0 and c == -p1[0]:
            x = -c
            for i, segment in enumerate(shape):
                if i == 0:
                    continue
                if (segment[0][0] <= x and x <= segment[1][0]) or (segment[1][0] <= x and x <= segment[0][0]):
                    x, y = self._twoPointCut((a, b, c), (self._lineFromTwoPoints(segment[0], segment[1])))
                    if self._isCutPossible((x, y), p1, normal):
                        cut_count += 1
                        cut_points.append((x, y))
        else:
            for i, segment in enumerate(shape):
                if i == 0:
                    continue

                if (a*segment[0][0] + b*segment[0][1] + c)*(a*segment[1][0] + b*segment[1][1] + c) <= 0:
                    x, y = self._twoPointCut((a, b, c), (self._lineFromTwoPoints(segment[0], segment[1])))
                    if self._isCutPossible((x, y), p1, normal):
                        cut_count += 1
                        cut_points.append((x, y))

        if (cut_count % 2) == 0:
            return True

        return self._find_point_in_polygon(p1, cut_points)

    def _find_point_in_polygon(self, base_point, cut_points):
        closest_point = None
        shortest_distance = None

        for cut_point in cut_points:
            distance = self.__distance(base_point, cut_point)
            if shortest_distance is None or distance < shortest_distance:
                shortest_distance = distance
                closest_point = cut_point

        result = np.asarray([(base_point[0] + closest_point[0])/2, (base_point[1] + closest_point[1])/2])

        return result

    def __distance(self, p1, p2):
        return np.linalg.norm(p2 - p1)

    def _lineFromTwoPoints(self, p1, p2):
        if (p1 == p2).all():
            raise ValueError("Points are not distinct.")
        else:
            x1, y1 = p1[0], p1[1]
            x2, y2 = p2[0], p2[1]

            if x1 == x2:
                return 1, 0, -x1
            else:
                m = (y2-y1)/(x2-x1)
                return -m, 1.0, (m * x1 - y1)

    def _twoPointCut(self, line1, line2):
        a1, b1, c1 = line1
        a2, b2, c2 = line2

        w = a1*b2 - a2*b1

        if w == 0:
            return False, False

        w_x = -c1*b2 - (-c2)*b1
        w_y = a1*(-c2) - a2*(-c1)

        return w_x/w, w_y/w

    def _isCutPossible(self, point1, point2, normal):
        if point1[0] is False and point1[1] is False:
            return False

        is_x_legit = False
        is_y_legit = False

        if (0 <= normal[0] and point2[0] <= point1[0]) or (0 >= normal[0] and point2[0] >= point1[0]):
            is_x_legit = True
        if (0 <= normal[1] and point2[1] <= point1[1]) or (0 >= normal[1] and point2[1] >= point1[1]):
            is_y_legit = True

        if is_x_legit and is_y_legit:
            return True

        return False
    
    def _calculateClosingPolygons(self, shapes):
        indexed_vertices = []
        indexed_sections = []
        holes = []

        # Old version - might be useful
        # curr_base = 0
        # for shape in shapes:
        #     if shape['fill'] is not True:
        #         holes.append(shape['fill'])
        #
        #     start = curr_base
        #     for segment in shape['segments']:
        #         indexed_vertices.append(segment[0])
        #         indexed_sections.append([curr_base, curr_base+1])
        #         curr_base += 1
        #     indexed_sections[-1][1] = start

        for shape in shapes:
            if shape['fill'] is not True:
                holes.append(shape['fill'])

            for segment in shape['segments']:
                indexed_sections.append(self.__getIndexedSegement(indexed_vertices, segment[0], segment[1]))

        t_data = dict()
        t_data['vertices'] = np.array(indexed_vertices)
        t_data['segments'] = np.array(indexed_sections)

        if holes:
            t_data['holes'] = np.array(holes)

        self._debug_drawShapes(t_data)

        t = triangle.triangulate(t_data, 'p')

        new_polygons = t['triangles']
        reindexed_vertices = t['vertices']

        close_polygons = []
        for polygon in new_polygons:
            close_polygons.append(reindexed_vertices[polygon[0]])
            close_polygons.append(reindexed_vertices[polygon[1]])
            close_polygons.append(reindexed_vertices[polygon[2]])


        self.__debug_drawClosedShapes(t_data, t)

        return close_polygons

    def getIndexedData(self, segments):
        indexed_vertices = []
        indexed_segments = []

        for segment in segments:
            indexed_segments.append(self.__getIndexedSegement(indexed_vertices,
                                                              np.array(segment[0], dtype=np.float32),
                                                              np.array(segment[1], dtype=np.float32)))

        return {'vertices': indexed_vertices, 'segments': indexed_segments}

    def __getIndexedSegement(self, indexed_vertices, segment_0, segment_1):
        if indexed_vertices:
            if (indexed_vertices[-1] == segment_0).all():
                indexed_segment_0 = len(indexed_vertices) - 1
            else:
                indexed_segment_0 = self.__findIndex(indexed_vertices, segment_0)
                if indexed_segment_0 == -1:
                    indexed_vertices.append(segment_0)
                    indexed_segment_0 = len(indexed_vertices) - 1

            indexed_segment_1 = self.__findIndex(indexed_vertices, segment_1)
            if indexed_segment_1 == -1:
                indexed_vertices.append(segment_1)
                indexed_segment_1 = len(indexed_vertices) - 1
        else:
            indexed_vertices.append(segment_0)
            indexed_vertices.append(segment_1)

            indexed_segment_0 = 0
            indexed_segment_1 = 1

        return [indexed_segment_0, indexed_segment_1]

    def __findIndex(self, list, vertex):
        for index in range(0, len(list)):
            if self._v_eq(list[index], vertex):
                return index

        return -1

    # Assumption -> There are two vertices on opposite sides
    def _calculateSplittedPolygons(self, verticies, verticies_sides):
        if verticies_sides[0] == Side.OnPlane or verticies_sides[1] == Side.OnPlane or verticies_sides[2] == Side.OnPlane:
            return self._calculateNewPolygonOneOnPlane(verticies, verticies_sides)

        if verticies_sides[0] == verticies_sides[1]:
            return self._calculateNewPolygonNoPlane(verticies[0], verticies[1], verticies[2])
        if verticies_sides[0] == verticies_sides[2]:
            return self._calculateNewPolygonNoPlane(verticies[2], verticies[0], verticies[1])
        if verticies_sides[1] == verticies_sides[2]:
            return self._calculateNewPolygonNoPlane(verticies[1], verticies[2], verticies[0])

    # Assumption -> There are two vertices on opposite sides and one on the plane (shit happens otherwise)
    def _calculateNewPolygonOneOnPlane(self, verticies, verticies_sides):
        if verticies_sides[0] == Side.OnPlane:
            cut_vert = self._calculateCutPoint(verticies[1], verticies[2])
            first_vert = verticies[1]
            first_polygon = [verticies[0], verticies[1], cut_vert]
            second_polygon = [verticies[0], cut_vert, verticies[2]]
        elif verticies_sides[1] == Side.OnPlane:
            cut_vert = self._calculateCutPoint(verticies[0], verticies[2])
            first_vert = verticies[0]
            first_polygon = [verticies[0], verticies[1], cut_vert]
            second_polygon = [verticies[1], verticies[2], cut_vert]
        elif verticies_sides[2] == Side.OnPlane:
            cut_vert = self._calculateCutPoint(verticies[0], verticies[1])
            first_vert = verticies[0]
            first_polygon = [verticies[0], cut_vert, verticies[2]]
            second_polygon = [cut_vert, verticies[1], verticies[2]]

        if self.__value <= first_vert[self.__axis]:
            return second_polygon, first_polygon
        return first_polygon, second_polygon

    def _calculateCutPoint(self, vert_1, vert_2):
        if vert_2[self.__axis] < vert_1[self.__axis]:  # Same result but floats...
            vert_1, vert_2 = vert_2, vert_1

        k = vert_2 - vert_1

        param1 = self.__axis
        param2 = (self.__axis + 1) % 3
        param3 = (self.__axis + 2) % 3

        cut_point = [0, 0, 0]  # x,y,z

        prop = (self.__value - vert_1[param1]) / (vert_2[param1] - vert_1[param1])
        cut_point[param1] = self.__value
        cut_point[param2] = prop * (vert_2[param2] - vert_1[param2]) + vert_1[param2]
        cut_point[param3] = prop * (vert_2[param3] - vert_1[param3]) + vert_1[param3]

        return np.asarray(cut_point, dtype=np.float32)

    def _calculateNewPolygonNoPlane(self, left_vert_1, left_vert_2, right_vert_3):
        vert_cut_1 = self._calculateCutPoint(left_vert_1, right_vert_3)
        vert_cut_2 = self._calculateCutPoint(left_vert_2, right_vert_3)
        single_polygon = [vert_cut_1, vert_cut_2, right_vert_3]

        double_polygon = [left_vert_1, left_vert_2, vert_cut_1, left_vert_2, vert_cut_2, vert_cut_1]

        if left_vert_1[self.__axis] < self.__value:
            return double_polygon, single_polygon
        return single_polygon, double_polygon

    def _bresenham_line(self, x, y, x2, y2):
        """Brensenham line algorithm"""
        steep = 0
        coords = []

        x, x2 = round(x), round(x2)
        y, y2 = round(y), round(y2)

        dx = abs(x2 - x)
        dy = abs(y2 - y)

        if (x2 - x) > 0:
            sx = 1
        else:
            sx = -1

        if (y2 - y) > 0:
            sy = 1
        else:
            sy = -1

        if dy > dx:
            steep = 1
            x, y = y, x
            dx, dy = dy, dx
            sx, sy = sy, sx

        d = (2 * dy) - dx
        for i in range(0, int(dx)):
            if steep:
                coords.append((y, x))
            else:
                coords.append((x, y))

            while d >= 0:
                y = y + sy
                d = d - (2 * dx)

            x = x + sx
            d = d + (2 * dy)

        coords.append((x2, y2))

        return coords

    def _delaunay_constrained_triangulation(self, indexed_data):
        t_data = dict()
        t_data['vertices'] = np.array(indexed_data['vertices'])
        t_data['segments'] = np.array(indexed_data['segments'])
        if indexed_data['holes']:
            t_data['holes'] = np.array(indexed_data['holes'])

        self._debug_drawShapes(t_data)

        t = triangle.triangulate(t_data, 'p')

        new_polygons = t['triangles']
        reindexed_vertices = t['vertices']

        return {'polygons': new_polygons, 'vertices': reindexed_vertices}

#########################################################
# Functions useful for debugging splitting
#########################################################

    # Remember to run it on main thread!
    def _debug_drawSections(self, sections, logprint: bool = False):
        if not self.__debug:
            return
        import triangle.plot as plot
        import matplotlib.pyplot as plt

        indexed_vertices = []
        indexed_sections = []

        # Prepare data for triangle
        curr_base = 0
        for segment in sections:
            if logprint:
                print(segment[0], " - ", segment[1])
            indexed_vertices.append(segment[0])
            indexed_vertices.append(segment[1])
            indexed_sections.append([curr_base, curr_base + 1])
            curr_base += 2

        t_data = dict()
        t_data['vertices'] = np.array(indexed_vertices)
        t_data['segments'] = np.array(indexed_sections)

        # Create plot
        plt.figure()

        ax1 = plt.subplot(121, aspect='equal')
        plot.plot(ax1, **t_data)

        plt.show()

    # Input in trinagle data - remember to run it on main thread!
    def _debug_drawShapes(self, shapes):
        if not self.__debug:
            return
        import triangle.plot as plot
        import matplotlib.pyplot as plt

        plt.figure()

        ax1 = plt.subplot(121, aspect='equal')
        plot.plot(ax1, **shapes)

        plt.show()

    # Input in trinagle data - remember to run it on main thread!
    def __debug_drawClosedShapes(self, shapes, closed_shapes):
        if not self.__debug:
            return
        import triangle.plot as plot
        import matplotlib.pyplot as plt

        plt.figure()

        ax1 = plt.subplot(121, aspect='equal')
        plot.plot(ax1, **shapes)

        ax2 = plt.subplot(122, sharex=ax1, sharey=ax1)
        triangle.plot.plot(ax2, **closed_shapes)

        plt.show()

    def __debug_printShapes(self, shapes):
        if not self.__debug:
            return

        for i, shape in enumerate(shapes):
            print("===================== SHAPE ", i, " ===========================")
            print("Sections: ", len(shape['segments']))
            print("Fill:     ", shape['fill'])
            # if len(shape) <= 2:
            #     print("INVALID")
            for segment in shape['segments']:
                print(segment[0], " - ", segment[1])
