import QtQuick 2.4
import QtQuick.Layouts 1.1
import Settings 1.0 as Settings

import "../singletons"
import "../tweaks"
import "../spinBox"

Item {
    id: button

    property alias buttonText: buttonText.text
    property alias buttonTextVisibility: buttonText.visible
    property alias buttonTextBackgroundHeight: textBackground.height
    property alias buttonWidth: button.width
    property alias buttonBackgroundColor: buttonBackground.color
    property alias deactivateButton: deactivateIndicator.visible
    property alias supportButton: supportButton
    property alias rigidButton: rigidButton

    property alias auto: autoText.visible
    property alias spinBox: spinBoxItem.visible
    property alias spinBoxUnit: spinBox.unitString

    property alias minValue: spinBox.minValue
    property alias maxValue: spinBox.maxValue
    property alias singleStep: spinBox.singleStep
    property alias acceptDot: spinBox.acceptDot
    property alias curaParam: spinBox.curaParam

    property alias autoMouseArea: autoMouseArea
    property alias undoMouseArea: undoMouseArea


    property string objectHelpText: "Set the minimum overhang angle for support generation manually. " +
                    "If the model rests on a very small buildplate area, it may break off " +
                    "which is why this parameter should be decreased in such a case. More " +
                    "supports mean longer printing time and more stable printout while fewer " +
                    "supports mean shorter printing time and less print stability. Support " +
                    "generation may be disabled by using the desired button.\n\n" +
                    "Rigid Support button changes the shape of the support to make it more stable but harder to remove."
    property string objectHelpImageSource: "../images/help/Support.png"


     ///temporary alias
    property alias spinBoxAlias: spinBox

    property real margin: (Globals.buttonHeight - Globals.buttonHeaderHeight - Globals.decoratorHeight - spinBox.height)/2

    Layout.preferredHeight: spinBoxItem.visible ? Globals.buttonHeight * 1.7: Globals.buttonHeight
    Layout.fillWidth: true

    ColumnLayout {
        id: layout
        anchors.fill: parent
        spacing: 0

        Rectangle {
            id: textBackground
            Layout.preferredHeight: Globals.buttonHeaderHeight
            Layout.fillWidth: true
            color: Colors.menuButton

            Text {
                id: buttonText
                anchors.centerIn: parent
                font.pixelSize: Globals.fontPixelSize
                font.capitalization: Font.AllUppercase
                font.family: Fonts.boxed

                color: Colors.buttonText
                text: "button"
            }
        }

        Rectangle {
            id: decorator
            anchors.top: textBackground.bottom
            Layout.fillWidth: true
            Layout.preferredHeight: Globals.decoratorHeight
            color: Colors.buttonDecorator
        }

        Rectangle {
            id: buttonBackground
            Layout.fillHeight: true
            Layout.fillWidth: true
            color: Colors.buttonBackground

            Item{
                anchors.fill: parent
                Text {
                    id: autoText
                    anchors.centerIn: parent
                    text: "Auto"
                    font.pixelSize: Globals.fontPixelSize
                    font.family: Fonts.boxed
                    color: Colors.buttonOptionText
                }

                MouseArea {
                    id: autoMouseArea
                    visible: autoText.visible
                    anchors.fill: parent
                }
            }

            ColumnLayout{
                id: spinBoxItem
                anchors.fill: parent
                visible: false
                spacing: Globals.buttonHeight * 0.1

                Spacer {}

                MenuButton {
                    id: supportButton
                    heightFactor: 1.5
                    Layout.fillWidth: true
                    Layout.leftMargin: margin
                    Layout.rightMargin: margin
                    menuButtonText: "Default"
                    buttonClickedAction.onTriggered:
                    {
                        var state = Settings.Notifications.support_enable
 
                        if (state == "None" || state == "True"){
                            menuButtonText = "Support: Disabled"
                            Settings.Notifications.support_enable = "False"
                            spinBoxAlias.deactivate = true
                            rigidButton.deactivate = true
                        } else {
                            menuButtonText = "Support: Enabled"
                            Settings.Notifications.support_enable = "True"
                            spinBoxAlias.deactivate = false
                            rigidButton.deactivate = false
                        }
                    }

                    helpText: objectHelpText
                    helpImageSource: objectHelpImageSource
                }

                MenuButton {
                    id: rigidButton
                    heightFactor: 1.5
                    Layout.fillWidth: true
                    Layout.leftMargin: margin
                    Layout.rightMargin: margin
                    menuButtonText: "Rigid support: Disabled"
                    buttonClickedAction.onTriggered:
                    {
                        var state = Settings.Notifications.rigid_enable

                        if (state == "True"){
                            menuButtonText = "Rigid support: Disabled"
                            Settings.Notifications.rigid_enable = "False"
                        } else {
                            menuButtonText = "Rigid support: Enabled"
                            Settings.Notifications.rigid_enable = "True"
                        }
                    }

                   helpText: objectHelpText
                   helpImageSource: objectHelpImageSource
                }

                RowLayout{
                    id: rowLayout
                    // Layout.preferredHeight: 
                    Layout.preferredWidth: parent.width
                    Layout.leftMargin: margin
                    Layout.rightMargin: margin
                    spacing: parent.width * 0.05
                    CustomSpinBox {
                        id: spinBox
                        standalone: true
                        unitString: "°"
                        minValue: 1
                        maxValue: 89
                        singleStep: 1
                        acceptDot: false
                        curaParam: "support_angle"
                    }

                    Image {
                        Layout.preferredHeight: parent.height * 0.9
                        Layout.preferredWidth: height

                        fillMode: Image.PreserveAspectFit
                        mipmap: true

                        source: "../images/undo.svg"

                        MouseArea {
                            id: undoMouseArea
                            anchors.fill: parent
                        }
                    }
                }

                Spacer {}
            }
        }
    }

    //Non-active button layer
    Rectangle {
        id: deactivateIndicator
        anchors.fill: parent
        visible: false
        color: Colors.buttonDeactivator
    }

    BottomShadow {}

    ButtonTweaks {
        id: supportHelp
        z: -1
        hoverIndicatorEnabled: false
        helpText: objectHelpText
        helpImageSource: objectHelpImageSource
    }
}
