import QtQuick 2.0
import Gence 1.0 as Gence
import UM 1.2 as UM
import Cura 1.0 as Cura

import "../singletons"
import "../tweaks"

Rectangle {
    id: backgroundRect
    width: parent.width
    height: Globals.fontPixelSize * 1.5
    color: selected ? Colors.customListSelectedBackground : Colors.customListBackground

    MouseArea {
        z: -1
        anchors.fill: parent
        onClicked: {
            if (mouse.modifiers && Qt.ControlModifier){
                Gence.ModelsHolder.printableNodes.toggleSelection(index)
            } else {
                Gence.ModelsHolder.printableNodes.handleSingleSelection(index)
            }
        }
    }

    Text {
        id: text
        height: parent.height
        anchors.left: parent.left
        anchors.leftMargin: layout.width * 0.02
        anchors.right: extruderSelector.visible ? extruderSelector.left : parent.right
        anchors.rightMargin: layout.width * 0.02
        font.pixelSize: Globals.fontPixelSize
        font.family: Fonts.boxed
        verticalAlignment: Text.AlignVCenter
        color: selected ? Colors.customListSelectedText : Colors.customListText
        text: name
        elide: Text.ElideRight
    }

    Rectangle {
        id: extruderSelector
        visible: multipleExtrudersProfile && Globals.multiextruderPrint
        anchors.right: parent.right
        anchors.rightMargin: Math.ceil(parent.height * 0.15)
        anchors.verticalCenter: parent.verticalCenter
        height: parent.height - Math.ceil(parent.height * 0.15)*2
        width: height

        border.width: 1
        border.color: Colors.customSettingsEditBorder

        Text {
            visible: parent.visible
            anchors.fill: parent
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter

            font.pixelSize: Globals.fontPixelSize * 0.8
            font.family: Fonts.boxed
            color: Colors.customListText

            text: {
                if(activeExtruder){
                    return activeExtruder
                }
                return "-"
            } 
        }

        MouseArea {
            z: 0
            visible: parent.visible
            width: backgroundRect.height
            height: width
            anchors.centerIn: parent
            preventStealing: true
            onClicked: {
                Gence.ModelsHolder.printableNodes.toggleExtruder(index)
            }
            ButtonTweaks {
                z: 0
                hoverIndicatorEnabled: false
                helpText: Help.multimaterialSwitch
            }
        }
    }
}