import QtQuick 2.4
import QtQuick.Layouts 1.1

import "../../buttons"
import Settings 1.0 as Settings

Item {
    property alias layoutHeight: layout.height
    property alias coolingSpinBox: coolingSpinBox

    ColumnLayout {
        anchors.fill: parent
        id: layout

        AdditionsSupportButton {
            id: supportSpinBox
            buttonText: "Support"
        }

        AdditionsButton {
            id: coolingSpinBox
            buttonText: "Cooling"
            spinBoxUnit: "%"
            minValue: 0
            maxValue: 100
            singleStep: 1
            curaParam: "cool_fan_speed"

            objectHelpText: "Set the printed model cooling manually. " +
                            "This value should be increased solely for " +
                            "printing thin components which must be nicer " +
                            "visually and do not need to be strong."
        }

        AdditionsButton {
            id: wallOffsetSpinBox
            buttonText: "Wall Offset"
            spinBoxUnit: " mm"
            minValue: -3.0
            maxValue: 3.0
            singleStep: 0.1
            precision: 2
            acceptDot: true
            curaParam: "xy_offset"

            objectHelpText:   "Changes the model size, extending or narrowing its " +
                        "walls regardless of its size. This figure presents the " +
                        "model loaded in the software (red) and after 3 mm offset " +
                        "has been applied (blue). It is recommended to improve the " +
                        "dimensional tolerance of the fitted parts."
            objectHelpImageSource: "../images/help/WallOffset.png"
        }

        AdditionsCompensationButton {
            id: xyCompensationSpinBox
            buttonText: "XY compensation"
            spinBoxUnit: "%"
        }
    }
}
