import QtQuick 2.4
import QtQuick.Controls 1.4
import UM 1.3 as UM
import Gence 1.0 as Gence
import "../../../JS/toolbox.js" as Toolbox

ScaleButtonContentForm {

    property bool nonUniformScale: false
    property double percentageJump: 0.01
    property double mmJump: 1.0


    Timer {
       id:timer
       interval: 20; running: true; repeat: false
    }

    Connections
    {
        target: timer
        onTriggered:
        {
            updateSpinBoxes()
            UM.ActiveTool.setProperty("NonUniformScale", nonUniformScale)
        }
    }

    Connections
    {
        id: onNodeScaled
        target: Gence.Actions.modelWasMoved
        onTriggered:
        {
            stopAllActions()
            updateSpinBoxes()
            startAllActions()
        }
    }


    onVisibleChanged: {
        if(visible)
        {
            onNodeScaled.target=Gence.Actions.modelWasMoved;
            timer.start()
        } else {
            onNodeScaled.target=null;
        }
    }

    function stopAllActions()
    {
        xSpinBox.topSpinBox.textField.onTextChanged.enabled=false
        xSpinBox.bottomSpinBox.textField.onTextChanged.enabled=false

        ySpinBox.topSpinBox.textField.onTextChanged.enabled=false
        ySpinBox.bottomSpinBox.textField.onTextChanged.enabled=false

        zSpinBox.topSpinBox.textField.onTextChanged.enabled=false
        zSpinBox.bottomSpinBox.textField.onTextChanged.enabled=false

    }
    function startAllActions()
    {
        xSpinBox.topSpinBox.textField.onTextChanged.enabled=true
        xSpinBox.bottomSpinBox.textField.onTextChanged.enabled=true

        ySpinBox.topSpinBox.textField.onTextChanged.enabled=true
        ySpinBox.bottomSpinBox.textField.onTextChanged.enabled=true

        zSpinBox.topSpinBox.textField.onTextChanged.enabled=true
        zSpinBox.bottomSpinBox.textField.onTextChanged.enabled=true
    }

    function updateSpinBoxes()
    {
        xSpinBox.topSpinBox.textField.text = Toolbox.round(100 * UM.ActiveTool.properties.getValue("ScaleX"), 2)
        ySpinBox.topSpinBox.textField.text = Toolbox.round(100 * UM.ActiveTool.properties.getValue("ScaleZ"), 2)
        zSpinBox.topSpinBox.textField.text = Toolbox.round(100 * UM.ActiveTool.properties.getValue("ScaleY"), 2)

        xSpinBox.bottomSpinBox.textField.text = Toolbox.round(UM.ActiveTool.properties.getValue("ObjectWidth"), 2)
        ySpinBox.bottomSpinBox.textField.text = Toolbox.round(UM.ActiveTool.properties.getValue("ObjectDepth"), 2)
        zSpinBox.bottomSpinBox.textField.text = Toolbox.round(UM.ActiveTool.properties.getValue("ObjectHeight"), 2)

    }

    Connections
    {
        target: uniformOnAction
        onTriggered: {
            nonUniformScale = !nonUniformScale
            UM.ActiveTool.setProperty("NonUniformScale", nonUniformScale)
            if(nonUniformScale)
                uniformOnButton.menuButtonText="Uniform off"
            else
                uniformOnButton.menuButtonText="Uniform on"
        }
    }

    Connections
    {

        target: resetAction
        onTriggered:
        {
            UM.ActiveTool.triggerAction("resetScale");
            stopAllActions();
            updateSpinBoxes();
            startAllActions();
        }
    }

    xSpinBox.topPlusAction.onClicked: setScale("ScaleX", UM.ActiveTool.properties.getValue("ScaleX") + percentageJump)
    ySpinBox.topPlusAction.onClicked: setScale("ScaleZ", UM.ActiveTool.properties.getValue("ScaleZ") + percentageJump)
    zSpinBox.topPlusAction.onClicked: setScale("ScaleY", UM.ActiveTool.properties.getValue("ScaleY") + percentageJump)

    xSpinBox.topMinusAction.onClicked: setScale("ScaleX", UM.ActiveTool.properties.getValue("ScaleX") - percentageJump)
    ySpinBox.topMinusAction.onClicked: setScale("ScaleZ", UM.ActiveTool.properties.getValue("ScaleZ") - percentageJump)
    zSpinBox.topMinusAction.onClicked: setScale("ScaleY", UM.ActiveTool.properties.getValue("ScaleY") - percentageJump)

    ySpinBox.bottomPlusAction.onClicked: setSize("ObjectDepth",UM.ActiveTool.properties.getValue("ObjectDepth") + mmJump)
    zSpinBox.bottomPlusAction.onClicked: setSize("ObjectHeight",UM.ActiveTool.properties.getValue("ObjectHeight") + mmJump)
    xSpinBox.bottomPlusAction.onClicked: setSize("ObjectWidth",UM.ActiveTool.properties.getValue("ObjectWidth") + mmJump)

    ySpinBox.bottomMinusAction.onClicked: setSize("ObjectDepth", UM.ActiveTool.properties.getValue("ObjectDepth") - mmJump)
    zSpinBox.bottomMinusAction.onClicked: setSize("ObjectHeight", UM.ActiveTool.properties.getValue("ObjectHeight") - mmJump)
    xSpinBox.bottomMinusAction.onClicked: setSize("ObjectWidth", UM.ActiveTool.properties.getValue("ObjectWidth") - mmJump)

    Connections
    {
        target: xSpinBox.onTopValueChanged
        onTriggered: setScale("ScaleX", xSpinBox.topValue/100.0)
    }
    Connections
    {
        target: ySpinBox.onTopValueChanged
        onTriggered: setScale("ScaleZ", ySpinBox.topValue/100.0)
    }
    Connections
    {
        target: zSpinBox.onTopValueChanged
        onTriggered: setScale("ScaleY", zSpinBox.topValue/100.0)
    }

    Connections
    {
        target: xSpinBox.onBottomValueChanged
        onTriggered: setSize("ObjectWidth", xSpinBox.bottomValue)
    }

    Connections
    {
        target: ySpinBox.onBottomValueChanged
        onTriggered: setSize("ObjectDepth", ySpinBox.bottomValue)
    }

    Connections
    {
        target: zSpinBox.onBottomValueChanged
        onTriggered: setSize("ObjectHeight", zSpinBox.bottomValue)
    }

    function setSize(dimensionString, value){
        if(value >= 0.1){
            UM.ActiveTool.setProperty(dimensionString, value);
        }
    }

    function setScale(axisString, value){
        var dimensionString = ""
        switch(axisString){
            case "ScaleX":
                dimensionString = "ObjectWidth"
                break
            case "ScaleY":
                dimensionString = "ObjectDepth"
                break
            case "ScaleZ":
                dimensionString = "ObjectHeight"
        }

        var originalSize = UM.ActiveTool.properties.getValue(dimensionString) / UM.ActiveTool.properties.getValue(axisString)

        if(originalSize * value >= 0.1){
            UM.ActiveTool.setProperty(axisString, value)
        }
    }
}
