import QtQuick 2.0
import QtQuick.Layouts 1.1

import "../../tweaks"
import "../../buttons"
import "../../singletons"

import Gence 1.0 as Gence
import Settings 1.0 as Settings

import UM 1.1 as UM

Item {
    id: rightMenuContent
    Layout.fillWidth: true

    signal defaultProfileLoaded()

    Component.onCompleted:
    {
        Settings.Notifications.setFunction("onPrinterChanged",Gence.Actions.onPrinterChanged.trigger)
        Settings.Notifications.setFunction("onNozzleChanged",Gence.Actions.onNozzleChanged.trigger)
    }

    Keys.onPressed:
    {
        event.accepted=false;
    }

    Connections {
        target: right_menu_manager
        onChangePrinterMenu: {
            printerButton.currentOption = value
            setButtonState(printButton, active)
        }
    }

    Connections {
        target: right_menu_manager
        onChangeMaterialMenu: {
            profileButton.currentOption = value
            setButtonState(profileButton, active)
        }
    }

    Connections {
        target: right_menu_manager
        onChangeQualityMenu: {
            qualityButton.currentOption = value
            setButtonState(qualityButton, active)
        }
    }

    Connections {
        target: right_menu_manager
        onChangeDurabilityMenu: {
            durabilityButton.currentOption = value
            setButtonState(durabilityButton, active)
        }
    }

    function setButtonState(button, active){
        switch(active){
            case "true":
                button.deactivateButton = false
                break
            case "false":
                button.deactivateButton = true
                break
        }
    }

    Connections
    {
        target: CuraApplication
        onDefaultProfileUsed:
        {
            enableQualityButtons()
        }
    }

    Connections
    {
        target: Gence.Actions.node_loaded
        onTriggered:
        {
            enablePrintButton()
            printButton.helpText =  "Prepare model for the print.\n\n" +
                                    "Software will calculate machine paths (g-code) for selected print settings. " +
                                    "The process progress bar will appear on the status bar. After the work is completed, " +
                                    "the software will enter the path preview mode to display the paths to be carried out by the printer."
        }
    }

    Connections
    {
        target: Gence.Actions.empty_plate
        onTriggered:
        {
            disablePrintButton()
        }
    }
    

    Connections
    {
        target: CuraApplication
        onCustomProfileLoaded:
        {
            disableQualityButtons()
        }
    }

    Connections
    {
        target:Gence.Actions.slicing
        onTriggered:
        {
            deactivateMenu()
            console.log ('slicing')
        }
    }

    Connections
    {
        target:Gence.Actions.deactivate_right_menu
        onTriggered:
        {
            deactivateMenu()
        }
    }

    Connections
    {
        target:Gence.Actions.activate_right_menu
        onTriggered:
        {
            enableAll()
            enablePrintButton()
        }
    }

    Connections
    {
        target:Gence.Actions.gcode_loading
        onTriggered:
        {
            deactivateMenu()
            console.log ('Loading GCode')
        }
    }

    Connections
    {
        target:Gence.Actions.sliced
        onTriggered: showSaveButton()
    }

    Connections
    {
        target:Gence.Actions.gcode_loaded
        onTriggered: {
            console.log('GCode Loaded')
        }
    }

    function showSaveButton(){
        enablePrintButton()
        printButton.buttonText="Save"
        printButton.helpText=   "Save the model as .gcode executable file which will " +
                                "be used by the 3D printer. After you select this button, " +
                                "a message will appear, allowing the user to choose location " +
                                "and a file name. If you use more than one model, the name will " +
                                "be generated based on the current date and time.\n\n"+
                                "It is recommended to save the file on a hard disk before copying it to the SD card."
    }

    Connections
    {
        target:Gence.Actions.abnormalUndo
        onTriggered:
        {
            printButton.buttonText="Prepare to print"
            enableAll()
            if(Gence.Notifications.appState !== Gence.Notifications.EMPTY_PLATE){
                enablePrintButton()
            }
        }
    }

    Connections
    {
        target:Gence.Actions.selectionEmpty
        onTriggered:
        {
            printerButton.closeMenu()
            profileButton.closeMenu()
            qualityButton.closeMenu()
            durabilityButton.closeMenu()
            additionsButton.closeMenu()
        }
    }

    ColumnLayout {
        anchors.fill: parent
        //previous value = ... * 0.014, changed 19.01.2017
        spacing: parent.height * 0.008

        RightMenuButton {
            id: printerButton
            buttonText: "Printer"
            buttonIcon: "../../images/printer.svg"
            buttonIconWhite: "../../images/printer_white.svg"
            contentLoaderSource: "../../buttons/content/PrinterButtonContent.qml"
            currentOption: "Empty"
            buttonAction.onClicked: printerButtonAction()
            menuWidth: width * 0.8
            // menuHeight: width * (0.18 + 0.1 * (printersCount + 1) + 0.03 * (printersCount)) + menuHeightMod
            menuHeight: Globals.emptyMenuHeight + content.item.printerListHeight

            helpText: Help.printerButton
            menuHelpText: Help.printerButton

        }

        RightMenuButton {
            id: profileButton
            buttonText: "Profile"
            buttonIcon: "../../images/profile.svg"
            buttonIconWhite: "../../images/profile_white.svg"
            contentLoaderSource: "../../buttons/content/ProfileButtonContent.qml"
            currentOption: "Empty"
            buttonAction.onClicked: profileButtonAction()
            menuWidth: width * 1.2
            menuHeight: Globals.emptyMenuHeight + menuHeightMod
            helpText: "Select the material to be used for the model."
            replacePlusChar: true
        }

        RightMenuButton {
            id: qualityButton
            buttonText: "Visual Quality"
            buttonIcon: "../../images/visual_quality.svg"
            buttonIconWhite: "../../images/visual_quality_white.svg"
            contentLoaderSource: "../../buttons/content/VisualQualityButtonContent.qml"
            currentOption: "Fine"
            buttonAction.onClicked: qualityButtonAction()
            menuWidth: width * 0.8
            menuHeight: Globals.emptyMenuHeight + content.item.qualitiesHeight

            helpText:   "Change the settings related to the visual quality of models. The delivered predefined settings " +
                        "have such options as layer height, print speed, work temperature, volume of extruded material per " +
                        "milimetre of path etc. selected specifically to achieve the required result. The selectable options " +
                        "in this menu change dynamically depending on the selected material.\n\n" +
                        "See 3dgence.com to get more detailed info about pre-defined printing profiles."
        }

        RightMenuButton {
            id: durabilityButton
            buttonText: "Durability"
            buttonIcon: "../../images/durability.svg"
            buttonIconWhite: "../../images/durability_white.svg"
            contentLoaderSource: "../../buttons/content/DurabilityButtonContent.qml"
            currentOption: "Solid"
            buttonAction.onClicked: durabilityButtonAction()
            menuWidth: width * 0.8
            menuHeight: Globals.emptyMenuHeight + content.item.qualitiesHeight

            helpText:   "Change the settings related to the durability of printed models. The pre-defined settings " +
                        "delivered include such options as density, volume of extruded material per millimetre of path, " +
                        "number of solid upper and lower layers in the model, print speed, tool temperature etc. selected " +
                        "specifically to achieve the required result. The selectable options in this menu change dynamically " +
                        "depending on the selected material and the Visual Quality option.\n\n" +
                        "See 3dgence.com to get more detailed info about pre-defined printing profiles."
        }


        RightMenuButton {
            id: additionsButton
            
            property bool modifyAdditionsMenuHeight: false

            buttonText: "Additions"
            buttonIcon: "../../images/additions.svg"
            buttonIconWhite: "../../images/additions_white.svg"
            contentLoaderSource: "../../buttons/content/AdditionsButtonContent.qml"
            currentOption: "Auto"
            buttonAction.onClicked: additionsButtonAction()
            menuWidth: width * 0.8
            menuHeight: width * 1.6 + (modifyAdditionsMenuHeight ? Globals.buttonHeight * 0.7 : 0)

            helpText:   "Quick access to advanced options impacting the model size (offsets), the minimum overhang angle in " +
                        "relation to the builplate to be used for the generating supports and cooling parameters. For most " +
                        "printouts this option should be set to Auto."
           // visible: Gence.Notifications.appState !== Gence.Notifications.SLICED
        }

        Spacer {
            Layout.preferredHeight: Globals.buttonHeight * 0.2
            Layout.fillHeight: false
        }

        ProfileDetailsButton {
            id: profileDetails
            visible: Gence.Notifications.appState !== Gence.Notifications.GCODE_LOADED && Gence.Notifications.appState !== Gence.Notifications.GCODE_LOADING
            helpText: "Detailed information about chosen material profile.\n\n" +
                      "To change printing parameters use the „Edit Custom Profile” option from the " +
                      "„Profile” menu. Copies of the all preinstalled material profiles are available " +
                      "as the custom profiles."
        }

        Spacer {}

        // LeftMenuButton {
        //     deactivateButton: true
        //     id: testButton
        //     buttonText: "test function"
	    //     buttonAction.onClicked: Globals.modalWindow.customSettingsWarning()

        //     cornerTriangleExists: false
        //     helpEnabled: false
        // }

        ActionButton {
            id: printButton
            deactivateButton: true
            buttonIcon: "../../images/logo_white.svg"
            buttonIconSizeRatio: 0.43
            buttonText: "Prepare to print"
            buttonAction.onClicked:
            {    
                if(Gence.Notifications.appState === Gence.Notifications.NODE_LOADED){
                    Gence.Actions.orderSlicing.trigger()
                }
                else if(Gence.Notifications.appState === Gence.Notifications.SLICED){// || Gence.Notifications.appState === Gence.Notifications.GCODE_LOADED){
                    UM.OutputDeviceManager.requestWriteToDevice(UM.OutputDeviceManager.activeDevice, Gence.Notifications.outputFilename, { "filter_by_machine": true, "limit_mimetypes": ['text/x-gcode']})
                }
            }
//            buttonAction.onClicked: customSignal()
        }
    }

    function printerButtonAction(){
        closeButtons()
        if(Printer.inCustom){
            Gence.CustomSettingsActions.loadLastProfile()
        }
        printerButton.openMenu()
    }

    function profileButtonAction(){
        closeButtons()
        profileButton.openMenu()
    }

    function qualityButtonAction(){
        closeButtons()
        qualityButton.openMenu()
    }

    function durabilityButtonAction(){
        closeButtons()
        durabilityButton.openMenu()
    }

    function additionsButtonAction(){
        closeButtons()
        additionsButton.openMenu()
    }

    function closeButtons(){
        printerButton.closeMenu()
        profileButton.closeMenu()
        qualityButton.closeMenu()
        durabilityButton.closeMenu()
        additionsButton.closeMenu()
    }

    function deactivateMenu(){
        closeButtons()
        disableAll()
        disablePrintButton()
    }

    function disableAll(){
        printerButton.deactivateButton=true
        profileButton.deactivateButton=true
        qualityButton.deactivateButton=true
        durabilityButton.deactivateButton=true
        additionsButton.deactivateButton=true
    }

    function enableAll(){
        if(!Printer.inCustom){
            qualityButton.deactivateButton=false
            durabilityButton.deactivateButton=false
        }
        printerButton.deactivateButton=false
        profileButton.deactivateButton=false
        additionsButton.deactivateButton=false
    }

    function disableQualityButtons(){
        qualityButton.deactivateButton=true
        durabilityButton.deactivateButton=true
        qualityButton.currentOption="Custom"
        durabilityButton.currentOption="Custom"
        profileButton.currentOption="Custom"
    }

    function enableQualityButtons(){
        printerButton.deactivateButton=false
        qualityButton.deactivateButton=false
        durabilityButton.deactivateButton=false
        defaultProfileLoaded()
    }

    function disablePrintButton(){
        printButton.deactivateButton=true
    }

    function enablePrintButton(){
        printButton.deactivateButton=false
    }
}
