import QtQuick.Layouts 1.1
import QtQuick 2.4

import "../singletons"

ColumnLayout {
    id: modalWindow

    property alias modalWindow: modalWindow
    property alias windowTitle: windowTitle.text
    property alias windowContent: contentLoader.source
    property alias modalWindowBorder: border

    property bool resizable: false

    property real loaderMarginsFactor: 0.04
    property real widthFactor: 6
    property real heightFactor: 6
    property real userWidth: 1
    property real userHeight: 1
    property real userX: 0
    property real userY: 0

    visible: false
    width: Math.min(base.width - x, Globals.buttonHeight * widthFactor * userWidth)
    height: Math.min(base.height - y, Globals.buttonHeight * heightFactor * userHeight)
    spacing: 0

    x: base.width * userX
    y: base.height * userY

    Rectangle {
        Layout.preferredHeight: Globals.buttonHeaderHeight
        Layout.fillWidth: true
        color: Colors.buttonTextBackground

        Text {
            id: windowTitle
            x: parent.height - contentHeight
            anchors.verticalCenter: parent.verticalCenter
            font.pixelSize: Globals.fontPixelSize
            font.capitalization: Font.AllUppercase
            font.family: Fonts.boxed

            color: Colors.buttonText
            text: "Window title"
        }

        Text {
            id: closeButton
            x: parent.width - (parent.height - contentHeight) - contentWidth
            anchors.verticalCenter: parent.verticalCenter
            font.pixelSize: Globals.fontPixelSize
            font.capitalization: Font.AllUppercase
            font.family: Fonts.boxed

            color: closeMouseArea.containsMouse ? Colors.buttonDecorator : Colors.buttonText
            text: "❌"
        }

        MouseArea {
            id: closeMouseArea
            anchors.fill: closeButton
            anchors.margins: -3
            hoverEnabled: true
            onClicked: forceClose()
        }

        MouseArea {
            z: -1
            anchors.fill: parent
            property point clickPos: Qt.point(1,1)
            // Update the mouse position
            onPressed: {
                clickPos = Qt.point(mouse.x, mouse.y);
            }
            // Move the whole component if mouse button has been pressed
            onPositionChanged: {
                var delta = Qt.point(mouse.x-clickPos.x, mouse.y-clickPos.y)

                if(modalWindow.x + delta.x > 0 && modalWindow.x + modalWindow.width + delta.x < base.width)
                    userX += delta.x / base.width
                else if (modalWindow.x + delta.x > 0)
                    userX = (base.width - modalWindow.width) / base.width
                else
                    userX = 0

                if(modalWindow.y + delta.y > 0 && modalWindow.y + modalWindow.height + delta.y < base.height)
                    userY += delta.y / base.height
                else if (modalWindow.y + delta.y > 0)
                    userY = (base.height - modalWindow.height) / base.height
                else
                    userY = 0
            }
        }
    }

    Rectangle {
        Layout.fillWidth: true
        Layout.preferredHeight: Globals.decoratorHeight
        color: Colors.buttonDecorator
    }

    Item {
        Layout.fillWidth: true
        Layout.fillHeight: true
        Rectangle {
            id: border
            anchors.fill: parent
            color: Colors.buttonMenuBorder
        }
        Rectangle {
            anchors.fill: parent
            anchors {
                bottomMargin: Globals.menuBorderWidth
                leftMargin: Globals.menuBorderWidth
                rightMargin: Globals.menuBorderWidth
            }
            color: Colors.buttonMenuBackground

            Loader {
                id: contentLoader
                anchors.fill: parent
                anchors.margins: parent.width * loaderMarginsFactor
            }
        }
        MouseArea {
            width: 5
            height: 5
            
            enabled: resizable
            visible: enabled

            anchors.right: parent.right
            anchors.bottom: parent.bottom
            cursorShape: Qt.SizeFDiagCursor

            property point clickPos: Qt.point(1, 1)
            onPressed: {
                clickPos = Qt.point(mouse.x, mouse.y);
            }
            // Move the whole component if mouse button has been pressed
            onPositionChanged: {
                var delta = Qt.point(mouse.x-clickPos.x, mouse.y-clickPos.y);
                var currentWidth = modalWindow.width
                var currentHeight = modalWindow.height

                if(delta.x + modalWindow.width <= base.width - modalWindow.x)
                    currentWidth += delta.x
                else {
                    currentWidth = base.width - modalWindow.x
                }

                if(delta.y + modalWindow.height <= base.height - modalWindow.y)
                    currentHeight += delta.y
                else {
                    currentHeight = base.height - modalWindow.y
                }

                userWidth = Math.max(1, currentWidth / (Globals.buttonHeight * widthFactor))
                userHeight = Math.max(1, currentHeight / (Globals.buttonHeight * heightFactor))
            }
        }
    }
}