// Copyright (c) 2016 Ultimaker B.V.
// Uranium is released under the terms of the AGPLv3 or higher.

import QtQuick 2.7
import QtQuick.Controls 1.1
import QtQuick.Controls.Styles 1.1

import Cura 1.0 as Cura

import "../../singletons"

SettingItem
{
    id: base

    contents: ComboBox
    {
        id: control

        model: Cura.ExtrudersModel
        {
            id: extruders_model
            onModelChanged: control.color = extruders_model.getItem(control.currentIndex).color
        }

        textRole: "name"

        anchors.fill: parent

        MouseArea
        {
            anchors.fill: parent
            acceptedButtons: Qt.NoButton
            onWheel: wheel.accepted = true;
        }

        style: ComboBoxStyle
        {
            font.family: Fonts.boxed
            font.pixelSize: Globals.fontPixelSize
            textColor: Colors.customListText
            selectionColor: Colors.customListSelectedBackground
            selectedTextColor: Colors.customListSelectedText
            padding.left: 0
            padding.right: 0

            background: Rectangle
            {
                color: enabled ? Colors.enabledBackground : Colors.disabledBackground
                border.width: 1
                border.color: Colors.customSettingsEditBorder
            }
            label: Item
            {
                Label
                {
                    anchors.left: parent.left
                    anchors.leftMargin: Globals.margin
                    anchors.right: downArrow.left
                    anchors.rightMargin: Globals.margin
                    anchors.verticalCenter: parent.verticalCenter

                    text: control.currentText;
                    font.family: Fonts.boxed
                    font.pixelSize: Globals.fontPixelSize
                    color: Colors.customListText

                    elide: Text.ElideRight;
                    verticalAlignment: Text.AlignVCenter;
                }

                Image {
                    id: downArrow
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.right: parent.right
                    anchors.rightMargin: Globals.margin
                    width: parent.height * 0.6
                    mipmap: true
                    fillMode: Image.PreserveAspectFit
                    source: "../../images/unfold.svg"
                }
            }
        }

        onActivated:
        {
            forceActiveFocus();
            propertyProvider.setPropertyValue("value", extruders_model.getItem(index).index);
        }

        onModelChanged: updateCurrentIndex()

        Binding
        {
            target: control
            property: "currentIndex"
            value:
            {
                for(var i = 0; i < extruders_model.rowCount(); ++i)
                {
                    if(extruders_model.getItem(i).index == propertyProvider.properties.value)
                    {
                        return i;
                    }
                }
                return -1;
            }
        }

        // In some cases we want to update the current color without updating the currentIndex, so it's a seperate function.
        function updateCurrentColor()
        {
            for(var i = 0; i < extruders_model.rowCount(); ++i)
            {
                if(extruders_model.getItem(i).index == currentIndex)
                {
                    control.color = extruders_model.getItem(i).color;
                    return;
                }
            }
        }

        function updateCurrentIndex()
        {
            for(var i = 0; i < extruders_model.rowCount(); ++i)
            {
                if(extruders_model.getItem(i).index == propertyProvider.properties.value)
                {
                    control.currentIndex = i;
                    return;
                }
            }
            currentIndex = -1;
        }
    }
}
