// Copyright (c) 2015 Ultimaker B.V.
// Uranium is released under the terms of the AGPLv3 or higher.

import QtQuick 2.1
import QtQuick.Layouts 1.1
import QtQuick.Controls 1.1
import QtQuick.Controls.Styles 1.1

import UM 1.1 as UM
import Cura 1.0 as Cura

import "../../singletons"

import "."

Item {
    id: base

    property alias contents: controlContainer.children

    property var showRevertButton: true
    property var doDepthIndentation: true
    property var doQualityUserSettingEmphasis: true

    // Create properties to put property provider stuff in (bindings break in qt 5.5.1 otherwise)
    property var state: propertyProvider.properties.state
    // There is no resolve property if there is only one stack.
    property var resolve: Cura.MachineManager.activeStackId != Cura.MachineManager.activeMachineId ? propertyProvider.properties.resolve : "None"
    property var stackLevels: propertyProvider.stackLevels
    property var stackLevel: stackLevels[0]

    Item
    {
        anchors.fill: parent

        Label
        {
            id: label

            anchors.left: parent.left
            anchors.leftMargin: doDepthIndentation ? definition.depth * Globals.margin : Globals.margin
            anchors.right: settingControls.left
            anchors.rightMargin: Globals.margin
            anchors.verticalCenter: parent.verticalCenter

            verticalAlignment: Text.AlignVCenter

            text: definition.label
            elide: Text.ElideMiddle

            color: Colors.customSettingsLabel
            opacity: propertyProvider.isValueUsed ? 1 : 0.5
            // emphasize the setting if it has a value in the user or quality profile
            font.italic: base.doQualityUserSettingEmphasis && base.stackLevel != undefined && base.stackLevel <= 1 ? true : false
            
            font.pixelSize: Globals.fontPixelSize
            font.family: Fonts.boxed
        }

        // TODO! Ask about other buttons and clean it up
        Row
        {
            id: settingControls

            height: parent.height / 2
            spacing: Globals.spacing

            anchors {
                right: controlContainer.left
                rightMargin: Globals.margin
                verticalCenter: parent.verticalCenter
            }

            UM.SimpleButton
            {
                id: revertButton

                visible: base.stackLevel == 0 && base.showRevertButton

                height: Globals.controlHeight / 2
                width: height

                anchors.verticalCenter: parent.verticalCenter

                color: Colors.revertButton
                hoverColor: Colors.revertButton

                iconSource: UM.Theme.getIcon("reset")

                onClicked: {
                    revertButton.focus = true
                    Cura.MachineManager.clearUserSettingAllCurrentStacks(propertyProvider.key)
                }
            }
        }

        Item
        {
            id: controlContainer

            enabled: propertyProvider.isValueUsed

            anchors.right: parent.right
            anchors.rightMargin: scrollActive ? 1 : Globals.margin
            anchors.verticalCenter: parent.verticalCenter
            width: parent.width * 0.5
            height: Globals.controlHeight
        }
    }
}
