// Copyright (c) 2015 Ultimaker B.V.
// Uranium is released under the terms of the AGPLv3 or higher.

import QtQuick 2.2
import QtQuick.Controls 1.2

import "../../singletons"

SettingItem
{
    id: base
    contents: Rectangle
    {
        id: control

        anchors.fill: parent

        border.width: 1
        border.color: Colors.customSettingsEditBorder

        color: {
            if (!enabled)
            {
                return Colors.disabledBackground
            }
            switch(propertyProvider.properties.validationState)
            {
                case "ValidatorState.Exception":
                    return Colors.errorBackground
                case "ValidatorState.MinimumError":
                    return Colors.errorBackground
                case "ValidatorState.MaximumError":
                    return Colors.errorBackground
                // case "ValidatorState.MinimumWarning":
                //     return Colors.warningBackground
                // case "ValidatorState.MaximumWarning":
                //     return Colors.warningBackground
                case "ValidatorState.Valid":
                    return Colors.enabledBackground

                default:
                    return Colors.enabledBackground
            }
        }

        Label
        {
            anchors.right: parent.right;
            anchors.rightMargin: Globals.margin
            anchors.verticalCenter: parent.verticalCenter

            text: definition.unit
            color: Colors.customSettingsUnit
            font.family: Fonts.boxed
            font.pixelSize: Globals.fontPixelSize
        }

        MouseArea
        {
            id: mouseArea
            anchors.fill: parent;
            //hoverEnabled: true;
            cursorShape: Qt.IBeamCursor
        }

        TextInput
        {
            id: input

            anchors
            {
                left: parent.left
                leftMargin: Globals.margin
                right: parent.right
                verticalCenter: parent.verticalCenter
            }

            Keys.onReleased:
            {
                propertyProvider.setPropertyValue("value", text)
            }

            onEditingFinished:
            {
                propertyProvider.setPropertyValue("value", text)
            }

            color: Colors.customSettingsText
            selectionColor: Colors.customListSelectedBackground
            selectedTextColor: Colors.customListSelectedText
            font.family: Fonts.boxed
            font.pixelSize: Globals.fontPixelSize

            selectByMouse: true

            maximumLength: 25

            validator: RegExpValidator { regExp: (definition.type == "int") ? /^-?[0-9]{0,10}$/ : /^-?[0-9]{0,9}[.,]?[0-9]{0,10}$/ } // definition.type property from parent loader used to disallow fractional number entry

            Binding
            {
                target: input
                property: "text"
                value:  {
                    // Stacklevels
                    // 0: user  -> unsaved change
                    // 1: quality changes  -> saved change
                    // 2: quality
                    // 3: material  -> user changed material in materialspage
                    // 4: variant
                    // 5: machine_changes
                    // 6: machine
                    if ((base.resolve != "None" && base.resolve) && (stackLevel != 0) && (stackLevel != 1)) {
                        // We have a resolve function. Indicates that the setting is not settable per extruder and that
                        // we have to choose between the resolved value (default) and the global value
                        // (if user has explicitly set this).
                        return base.resolve;
                    } else {
                        return propertyProvider.properties.value;
                    }
                }
                when: !input.activeFocus
            }
        }
    }
}
