# Copyright (c) 2015 Ultimaker B.V.
# Uranium is released under the terms of the AGPLv3 or higher.

from PyQt5.QtCore import QObject, pyqtSignal, pyqtProperty, pyqtSlot
from PyQt5.QtQml import QJSValue

from UM.Application import Application
from UM.Scene.Selection import Selection
from UM.Scene.SceneNode import SceneNode
from UM.Controller import Controller


class SelectionProxy(QObject):
    def __init__(self, parent=None):
        super().__init__(parent)
        Selection.selectionChanged.connect(self._onSelectionChanged)
        self.controller = Application.getInstance().getController()

    selectionChanged = pyqtSignal()
    newNodeSelected = pyqtSignal()
    # deafnessChanged = pyqtSignal()

    # @pyqtProperty(bool, notify=deafnessChanged)
    # def isDeaf(self):
    #     return self.controller.getSelectionTool().deaf

    # @isDeaf.setter
    # def isDeaf(self, deafness: bool):
    #     self.controller.getSelectionTool().deaf = deafness

    @pyqtProperty(bool, notify=selectionChanged)
    def hasSelection(self):
        return Selection.hasSelection()

    @pyqtProperty(bool, notify=selectionChanged)
    def hasSingleSelection(self):
        return Selection.hasSingleSelection()

    @pyqtProperty(str)
    def selectedNode(self):
        selectedName = ""
        if (Selection.hasSelection()):
            node = Selection.getSelectedObject(0)
            selectedName = node.getName();
        return selectedName

    @pyqtProperty("QVariantList")
    def selectedSize(self):
        selectedSize = [0, 0, 0]
        if (Selection.hasSelection()):
            node = Selection.getSelectedObject(0)
            box = node.getBoundingBox()
            selectedSize[0] = float(box.width)
            selectedSize[1] = float(box.depth)
            selectedSize[2] = float(box.height)

        return selectedSize

    @pyqtProperty("QVariantList")
    def nodesSize(self):
        return Selection.getSelectableObjectsBox()

    @pyqtSlot()
    def selectNext(self):
        Selection.cycleSelection()

    def _onSelectionChanged(self):
        self.selectionChanged.emit()


def createSelectionProxy(engine, script_engine):
    return SelectionProxy()
